# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

import json

from azure.cli.testsdk import (ResourceGroupPreparer, ScenarioTest)
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.core.azclierror import ResourceNotFoundError as CliResourceNotFoundError, MutuallyExclusiveArgumentError
from azure.cli.command_modules.appconfig.tests.latest._test_utils import create_config_store, CredentialResponseSanitizer, get_resource_name_prefix

class AppConfigSnapshotLiveScenarioTest(ScenarioTest):

    def __init__(self, *args, **kwargs):
        kwargs["recording_processors"] = kwargs.get("recording_processors", []) + [CredentialResponseSanitizer()]
        super().__init__(*args, **kwargs)


    @ResourceGroupPreparer(parameter_name_for_location='location')
    @AllowLargeResponse()
    def test_azconfig_snapshot_mgmt(self, resource_group, location):
        store_name_prefix = get_resource_name_prefix('SnapshotStore') 
        config_store_name = self.create_random_name(prefix=store_name_prefix, length=24)
        snapshot_name = "TestSnapshot"
        store_location = 'francecentral'
        sku = 'standard'

        self.kwargs.update({
            'config_store_name': config_store_name,
            'snapshot_name': snapshot_name,
            'rg_loc': store_location,
            'rg': resource_group,
            'sku': sku,
            'retention_days': 1,
            'enable_purge_protection': False
        })

        create_config_store(self, self.kwargs)

        credential_list =  self.cmd('appconfig credential list -n {config_store_name} -g {rg}').get_output_in_json()
        self.kwargs.update({
            'connection_string': credential_list[0]['connectionString']
        })

        entry_key = "TestKey1"
        entry_value = "TestValue1"
        entry_key2 = "TestKey2"
        entry_value2 = "TestValue2"
        dev_label = "dev"
        entry_key3 = "LastTestKey"
        entry_value3 = "LastTestValue"

        # Create 2 keys with a common prefix and label "dev"
        self.kwargs.update({
            "key": entry_key,
            "value": entry_value,
            "label": dev_label
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key),
                         self.check('value', entry_value),
                         self.check('label', dev_label)])

        self.kwargs.update({
            'key': entry_key2,
            'value': entry_value2,
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key2),
                         self.check('value', entry_value2),
                         self.check('label', dev_label)])

        self.kwargs.update({
            'key': entry_key3,
            'value': entry_value3,
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key3),
                         self.check('value', entry_value3),
                         self.check('label', dev_label)])

        # Create a snapshot of all key-values that begin with the prefix 'Test'
        filter_dict = { "key": "Test*", "label": dev_label, "tags": [] }
        retention_period = 3600 # Set retention period of 1 hour
        self.kwargs.update({
            'filter': '\'{}\''.format(json.dumps(filter_dict)),
            'retention_period': retention_period
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter} --retention-period {retention_period} --composition-type key_label --tags tag1=value1',
                 checks=[self.check('itemsCount', 2),
                         self.check('status', 'ready')])


        # Test showing created snapshot
        created_snapshot = self.cmd('appconfig snapshot show --connection-string {connection_string} --snapshot-name {snapshot_name} --fields name status items_count filters').get_output_in_json()

        self.assertEqual(created_snapshot['items_count'], 2)
        self.check(created_snapshot['status'], 'ready')
        self.assertDictEqual(created_snapshot['filters'][0], filter_dict)
        self.assertRaises(KeyError, lambda: created_snapshot['created'])

        # Test listing snapshots
        created_snapshots = self.cmd('appconfig snapshot list --snapshot-name {snapshot_name} --connection-string {connection_string} --fields name status items_count filters').get_output_in_json()
        self.assertEqual(created_snapshots[0]['items_count'], 2)
        self.assertEqual(created_snapshots[0]['status'], 'ready')
        self.assertDictEqual(created_snapshots[0]['filters'][0], filter_dict)

        # Test snapshot archive
        archived_snapshot = self.cmd('appconfig snapshot archive --connection-string {connection_string} --snapshot-name {snapshot_name}').get_output_in_json()
        self.assertIsNotNone(archived_snapshot['expires'])
        self.assertEqual(archived_snapshot['status'], 'archived')
        active_snapshots = self.cmd('appconfig snapshot list --connection-string {connection_string} --status ready').get_output_in_json()
        self.assertEqual(len(active_snapshots), 0)

        # Test snapshot recovery
        self.cmd('appconfig snapshot recover --connection-string {connection_string} -s {snapshot_name}',
                                     checks=[self.check('itemsCount', 2),
                                             self.check('status', 'ready'),
                                             self.check('expires', None),])
        archived_snapshots = self.cmd('appconfig snapshot list --connection-string {connection_string} --status archived').get_output_in_json()
        self.assertEqual(len(archived_snapshots), 0)

        # Test listing snapshot kvs
        kvs = self.cmd('appconfig kv list --connection-string {connection_string} --snapshot {snapshot_name}').get_output_in_json()
        assert len(kvs) == 2

        # Test error returned for listing kvs in non-existent snapshot
        non_existent_snapshot_name = "non_existent_snapshot"

        self.kwargs.update({
            'snapshot_name': non_existent_snapshot_name
        })

        with self.assertRaisesRegex(CliResourceNotFoundError, f'No snapshot with name \'{non_existent_snapshot_name}\' was found.'):
            self.cmd('appconfig kv list --connection-string {connection_string} --snapshot {snapshot_name}')

        # Test snapshot import/export
        config_store_2_name = self.create_random_name(prefix=store_name_prefix, length=24)

        self.kwargs.update({
            'config_store_name': config_store_2_name,
            'snapshot_name': snapshot_name,
        })

        create_config_store(self, self.kwargs)

        credential_list_2 =  self.cmd('appconfig credential list -n {config_store_name} -g {rg}').get_output_in_json()
        self.kwargs.update({
            'dest_connection_string': credential_list_2[0]['connectionString']
        })

        # Export snapshot kvs to store
        self.cmd('appconfig kv export -d appconfig --connection-string {connection_string} --dest-connection-string {dest_connection_string} --snapshot {snapshot_name} -y')

        # Export with skip-features should fail
        with self.assertRaisesRegex(MutuallyExclusiveArgumentError, "'--snapshot' cannot be specified with '--key',  '--label', '--skip-keyvault' or '--skip-features' arguments."):
            self.cmd('appconfig kv export -d appconfig --connection-string {connection_string} --dest-connection-string {dest_connection_string} --snapshot {snapshot_name} --skip-features -y')

        # Export with skip-keyvault should fail
        with self.assertRaisesRegex(MutuallyExclusiveArgumentError, "'--snapshot' cannot be specified with '--key',  '--label', '--skip-keyvault' or '--skip-features' arguments."):
            self.cmd('appconfig kv export -d appconfig --connection-string {connection_string} --dest-connection-string {dest_connection_string} --snapshot {snapshot_name} --skip-keyvault -y')

        # List snapshots in store
        dest_kvs = self.cmd('appconfig kv list --connection-string {dest_connection_string} --key * --label *').get_output_in_json()
        self.assertEqual(len(dest_kvs), 2)

        # Delete all kvs
        self.cmd('appconfig kv delete --connection-string {dest_connection_string} --key * --label * -y')

        # Import snapshot kvs from source
        self.cmd('appconfig kv import -s appconfig --connection-string {dest_connection_string} --src-connection-string {connection_string} --src-snapshot {snapshot_name} -y')

        # Import with skip-features should fail
        with self.assertRaisesRegex(MutuallyExclusiveArgumentError, "'--src-snapshot' cannot be specified with '--src-key', '--src-label', or '--skip-features' arguments."):
            self.cmd('appconfig kv import -s appconfig --connection-string {dest_connection_string} --src-connection-string {connection_string} --src-snapshot {snapshot_name} --skip-features -y')

        # List snapshots in store
        current_kvs = self.cmd('appconfig kv list --connection-string {dest_connection_string} --key * --label *').get_output_in_json()
        self.assertEqual(len(current_kvs), 2)


    @ResourceGroupPreparer(parameter_name_for_location='location')
    @AllowLargeResponse()
    def test_azconfig_snapshot_filtering(self, resource_group, location):
        store_name_prefix = get_resource_name_prefix('SnapshotFilters') 
        config_store_name = self.create_random_name(prefix=store_name_prefix, length=36)
        store_location = 'francecentral'
        sku = 'standard'

        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': store_location,
            'rg': resource_group,
            'sku': sku,
            'retention_days': 1,
            'enable_purge_protection': False
        })

        create_config_store(self, self.kwargs)

        credential_list =  self.cmd('appconfig credential list -n {config_store_name} -g {rg}').get_output_in_json()
        self.kwargs.update({
            'connection_string': credential_list[0]['connectionString']
        })

        # # Filter by tags test

        # Set key-value with 5 tags
        key_with_tags = "KeyWithTags"
        label_with_tags = "labelWithTags"
        retention_period = 3600 # Set retention period of 1 hour
        tags = {
            "tag1": "value1",
            "tag2": "value2",
            "tag3": "value3",
            "tag4": "value4",
            "tag5": "value5"
        }
        tags_list = [f"{k}={v}" for k, v in tags.items()]
        tags_str = ' '.join(tags_list)

        self.kwargs.update({
            'key': key_with_tags + "1",
            'label': label_with_tags,
            'tags': tags_str,
            'retention_period': retention_period
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tags} -y',
             checks=[self.check('key', key_with_tags + "1"),
             self.check('label', label_with_tags),
            self.check('tags', tags)])

        # Create snapshot with 5 tags filter
        key_with_tags_filter = key_with_tags + "*"
        filter_with_tags = { "key": key_with_tags_filter, "label": label_with_tags, "tags": tags_list }
        self.kwargs.update({
            'filter_with_tags': '\'{}\''.format(json.dumps(filter_with_tags)),
            'snapshot_name': "TestSnapshotWith5Tags"
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter_with_tags} --retention-period {retention_period} --composition-type key_label',
             checks=[self.check('itemsCount', 1),
                 self.check('status', 'ready')])

        # Set key-value with tag1 only
        tag1_dict = {"tag1": "value1"}
        tag1_list = [f"{k}={v}" for k, v in tag1_dict.items()]
        tag1 = ' '.join(tag1_list)
        self.kwargs.update({
            'key': key_with_tags + "2",
            'label': label_with_tags,
            'tag1': tag1
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tag1} -y',
             checks=[self.check('key', key_with_tags + "2"),
                 self.check('label', label_with_tags),
                 self.check('tags', tag1_dict)])

        # Create snapshot with tag1 filter
        filter_with_tag1 = { "key": key_with_tags_filter, "label": label_with_tags, "tags": tag1_list }
        self.kwargs.update({
            'filter_with_tag1': '\'{}\''.format(json.dumps(filter_with_tag1)),
            'snapshot_name': "TestSnapshotWith1Tag"
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter_with_tag1} --retention-period {retention_period} --composition-type key_label',
             checks=[self.check('itemsCount', 2),
                 self.check('status', 'ready')])

        # Set key-value with empty tag value
        empty_tag_value_dict = {"tag1": ""}
        tag_with_empty_value_list = [f"{k}={v}" for k, v in empty_tag_value_dict.items()]
        tag_with_empty_value = ' '.join(tag_with_empty_value_list)
        self.kwargs.update({
            'key': key_with_tags + "3",
            'label': label_with_tags,
            'tag_with_empty_value': tag_with_empty_value
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tag_with_empty_value} -y',
             checks=[self.check('key', key_with_tags + "3"),
                 self.check('label', label_with_tags),
                 self.check('tags', empty_tag_value_dict)])

        # Create snapshot with empty tag value filter
        filter_with_empty_tag_value = { "key": key_with_tags_filter, "label": label_with_tags, "tags": tag_with_empty_value_list }
        self.kwargs.update({
            'filter_with_empty_tag_value': '\'{}\''.format(json.dumps(filter_with_empty_tag_value)),
            'snapshot_name': "TestSnapshotWithEmptyTagValue"
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter_with_empty_tag_value} --retention-period {retention_period} --composition-type key_label',
             checks=[self.check('itemsCount', 1),
             self.check('status', 'ready')])

        # Create snapshot with key-values with any tags
        filter_all_kvs = { "key": key_with_tags_filter, "label": label_with_tags }
        self.kwargs.update({
            'filter_all_kvs': '\'{}\''.format(json.dumps(filter_all_kvs)),
            'snapshot_name': "TestSnapshotWithAllKVs"
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter_all_kvs} --retention-period {retention_period} --composition-type key_label',
             checks=[self.check('itemsCount', 3),
                 self.check('status', 'ready')])

        entry_key = "TestKey1"
        entry_value = "TestValue1"
        entry_key2 = "TestKey2"
        entry_value2 = "TestValue2"
        dev_label = "dev"
        entry_key3 = "LastTestKey"
        entry_value3 = "LastTestValue"

        # Create 2 keys with a common prefix and label "dev"
        self.kwargs.update({
            "key": entry_key,
            "value": entry_value,
            "label": dev_label
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key),
                         self.check('value', entry_value),
                         self.check('label', dev_label)])

        self.kwargs.update({
            'key': entry_key2,
            'value': entry_value2,
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key2),
                         self.check('value', entry_value2),
                         self.check('label', dev_label)])

        self.kwargs.update({
            'key': entry_key3,
            'value': entry_value3,
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --label {label} -y',
                 checks=[self.check('key', entry_key3),
                         self.check('value', entry_value3),
                         self.check('label', dev_label)])

        # Create a snapshot of all key-values that begin with the prefix 'Test'
        filter_dict = { "key": "Test*", "label": dev_label }
        self.kwargs.update({
            'filter': '\'{}\''.format(json.dumps(filter_dict)),
            'retention_period': retention_period,
            'snapshot_name': "TestSnapshotWithPrefix"
        })

        self.cmd('appconfig snapshot create --connection-string {connection_string} --snapshot-name {snapshot_name} --filters {filter} --retention-period {retention_period} --composition-type key_label --tags tag1=value1',
                 checks=[self.check('itemsCount', 2),
                         self.check('status', 'ready')])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, record_only, StorageAccountPreparer
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.core.exceptions import HttpResponseError, ResourceExistsError


class TestLogProfileScenarios(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace', location='centralus')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_default(self, resource_group):
        self.kwargs.update({
            'name': self.create_random_name('clitest', 20)
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --tags clitest=myron", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'PerGB2018')
        ])

        self.cmd("monitor log-analytics workspace update -g {rg} -n {name} --retention-time 100", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 100)
        ])

        self.cmd("monitor log-analytics workspace show -g {rg} -n {name}", checks=[
            self.check('retentionInDays', 100)
        ])

        self.cmd("monitor log-analytics workspace list-usages -g {rg} -n {name}")
        self.cmd("monitor log-analytics workspace list -g {rg}", checks=[
            self.check('length(@)', 1),
        ])
        self.cmd("monitor log-analytics workspace get-shared-keys -g {rg} -n {name}", checks=[
            self.check("contains(keys(@), 'primarySharedKey')", True),
            self.check("contains(keys(@), 'secondarySharedKey')", True)
        ])

        self.cmd("monitor log-analytics workspace get-schema -g {rg} -n {name}")

        self.cmd("monitor log-analytics workspace pack enable -g {rg} --workspace-name {name} -n AzureSecurityOfThings")
        self.cmd("monitor log-analytics workspace pack list -g {rg} --workspace-name {name}", checks=[
            self.check("@[?name=='AzureSecurityOfThings'].enabled", '[True]')
        ])

        self.cmd(
            "monitor log-analytics workspace pack disable -g {rg} --workspace-name {name} -n AzureSecurityOfThings")
        self.cmd("monitor log-analytics workspace pack list -g {rg} --workspace-name {name}", checks=[
            self.check("@[?name=='AzureSecurityOfThings'].enabled", '[False]')
        ])

        # test list-management-groups
        self.cmd("monitor log-analytics workspace list-management-groups -g {rg} -n {name}", checks=[
            self.check('length(@)', 0)
        ])

        self.cmd("monitor log-analytics workspace delete -g {rg} -n {name} -y")

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_update_sku', location='westus')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_update_sku(self, resource_group):
        workspace_name = self.create_random_name('clitest', 20)
        self.kwargs.update({
            'name': workspace_name
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --quota 1 --sku PerGB2018", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'PerGB2018'),
            self.check('workspaceCapping.dailyQuotaGb', 1.0)
        ])

        self.cmd("monitor log-analytics workspace update -g {rg} -n {name} --quota 2 --level 100 --sku CapacityReservation", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('sku.name', 'CapacityReservation'),
            self.check('sku.capacityReservationLevel', 100),
            self.check('workspaceCapping.dailyQuotaGb', 2.0)
        ])

        self.cmd("monitor log-analytics workspace delete -g {rg} -n {name} -y")

    @record_only()
    def test_monitor_log_analytics_workspace_failover_failback(self):
        self.kwargs.update({
            'name': "clitestfailoverfailback",
            'rg': "workspace-test"
        })
        # workspace enabled with replication cannot be processed in 24 hours, manage this resource manually, resource group in westus, test failover and failback api only
        # self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --tags clitest=liwa --replication-enabled 1 --replication-location eastus", checks=[
        #     self.check('provisioningState', 'Succeeded'),
        #     self.check('tags.clitest', 'liwa'),
        #     self.check('replication.enabled', True),
        #     self.check('replication.location', 'eastus')
        # ])

        self.cmd("monitor log-analytics workspace show -g {rg} -n {name}", checks=[
            self.check('tags.clitest', 'liwa'),
            self.check('replication.enabled', True),
            self.check('replication.location', 'eastus')
        ])

        # following error message is given from servic side based on service business logic and might change at runtime
        with self.assertRaisesRegex(HttpResponseError, "Workspace failover can't be triggered at this time because replication was enabled less than 24 hours ago."):
            self.cmd('monitor log-analytics workspace failover -g {rg} --workspace-name {name} --location eastus')

        with self.assertRaisesRegex(ResourceExistsError, "Failover is not active for this workspace."):
            self.cmd("monitor log-analytics workspace failback -g {rg} --workspace-name {name}")

        with self.assertRaisesRegex(HttpResponseError, " Workspace replication cannot be disabled at this time because it was enabled in the last hour"):
            self.cmd("monitor log-analytics workspace update -g {rg} -n {name} --replication-enabled 0")

        with self.assertRaisesRegex(ResourceExistsError, "Workspace '{0}' cannot be deleted because it is linked to DataCollectionEndpoint".format(self.kwargs['name'])):
            self.cmd("monitor log-analytics workspace delete -g {rg} -n {name} -y")

    @record_only()
    def test_monitor_log_analytics_workspace_linked_service_common_scenario(self):
        cluster_resource_id_1 = '/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272f09590/resourceGroups/azure-cli-test' \
                                '-scus/providers/Microsoft.OperationalInsights/clusters/yu-test-cluster1'
        recorded_cluster_resource_id_1 = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/azure' \
                                         '-cli-test-scus/providers/Microsoft.OperationalInsights/clusters/yu-test' \
                                         '-cluster1'
        cluster_resource_id_2 = '/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272f09590/resourceGroups/azure-cli-test' \
                                '-scus/providers/Microsoft.OperationalInsights/clusters/yu-test-cluster2'
        recorded_cluster_resource_id_2 = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/azure' \
                                         '-cli-test-scus/providers/Microsoft.OperationalInsights/clusters/yu-test' \
                                         '-cluster2'
        linked_service_name = 'cluster'

        self.kwargs.update({
            'rg': 'azure-cli-test-scus',
            'workspace_name': 'yu-test-ws1',
            'linked_service_name': linked_service_name,
            'cluster_resource_id_1': cluster_resource_id_1,
            'cluster_resource_id_2': cluster_resource_id_2
        })

        self.cmd("monitor log-analytics workspace linked-service create -g {rg} --workspace-name {workspace_name} "
                 "-n {linked_service_name} --write-access-resource-id {cluster_resource_id_1}",
                 checks=[
                     self.check('provisioningState', 'Succeeded'),
                     self.check('writeAccessResourceId', recorded_cluster_resource_id_1)
                 ])

        self.cmd("monitor log-analytics workspace linked-service update -g {rg} --workspace-name {workspace_name} "
                 "-n {linked_service_name} --write-access-resource-id {cluster_resource_id_2}",
                 checks=[
                     self.check('provisioningState', 'Succeeded'),
                     self.check('writeAccessResourceId', recorded_cluster_resource_id_2)
                 ])

        self.cmd("monitor log-analytics workspace linked-service show -g {rg} --workspace-name {workspace_name} "
                 "-n {linked_service_name}",
                 checks=[
                     self.check('provisioningState', 'Succeeded'),
                     self.check('writeAccessResourceId', recorded_cluster_resource_id_2)
                 ])

        self.cmd("monitor log-analytics workspace linked-service list -g {rg} --workspace-name {workspace_name}",
                 checks=[self.check('length(@)', 1)])

        self.cmd("monitor log-analytics workspace linked-service delete -g {rg} --workspace-name {workspace_name} "
                 "-n {linked_service_name} -y", checks=[])

        self.cmd("monitor log-analytics workspace linked-service list -g {rg} --workspace-name {workspace_name}",
                 checks=[self.check('length(@)', 0)])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_linked_storage', location='eastus')
    @AllowLargeResponse()
    @StorageAccountPreparer(name_prefix='saws1', kind='StorageV2', sku='Standard_LRS', parameter_name='account_1',
                            location='eastus')
    @StorageAccountPreparer(name_prefix='saws2', kind='StorageV2', sku='Standard_LRS', parameter_name='account_2',
                            location='eastus')
    @StorageAccountPreparer(name_prefix='saws3', kind='StorageV2', sku='Standard_LRS', parameter_name='account_3',
                            location='eastus')
    @StorageAccountPreparer(name_prefix='saws4', kind='StorageV2', sku='Standard_LRS', parameter_name='account_4',
                            location='eastus')
    def test_monitor_log_analytics_workspace_linked_storage(self, resource_group, account_1,
                                                            account_2, account_3, account_4):
        from azure.mgmt.core.tools import resource_id
        self.kwargs.update({
            'name': self.create_random_name('clitest', 20),
            'name_2': self.create_random_name('clitest', 20),
            'rg': resource_group,
            'sa_1': account_1,
            'sa_2': account_2,
            'sa_3': account_3,
            'sa_4': account_4,
            'sa_id_1': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=account_1,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'sa_id_2': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=account_2,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'sa_id_3': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=account_3,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'sa_id_4': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=account_4,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --tags clitest=myron", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'PerGB2018')
        ])

        self.cmd('monitor log-analytics workspace linked-storage create '
                 '--type CustomLogs -g {rg} -n {name} --storage-accounts {sa_1}',
                 checks=[
                     self.check('storageAccountIds[0]', '{sa_id_1}'),
                     self.check('name', 'CustomLogs')
                 ])

        self.cmd('monitor log-analytics workspace linked-storage add '
                 '--type CustomLogs -g {rg} -n {name} --storage-accounts {sa_2} {sa_id_3}',
                 checks=[
                     self.check('storageAccountIds[0]', '{sa_id_1}'),
                     self.check('storageAccountIds[1]', '{sa_id_2}'),
                     self.check('storageAccountIds[2]', '{sa_id_3}')
                 ])

        self.cmd('monitor log-analytics workspace linked-storage remove '
                 '--type CustomLogs -g {rg} -n {name} --storage-accounts {sa_1}',
                 checks=[
                     self.check('storageAccountIds[0]', '{sa_id_2}'),
                     self.check('storageAccountIds[1]', '{sa_id_3}')
                 ])

        self.cmd('monitor log-analytics workspace linked-storage show '
                 '--type CustomLogs -g {rg} -n {name}',
                 checks=[
                     self.check('storageAccountIds[0]', '{sa_id_2}'),
                     self.check('storageAccountIds[1]', '{sa_id_3}')
                 ])

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name_2} --tags clitest=myron", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'PerGB2018')
        ])

        self.cmd('monitor log-analytics workspace linked-storage create '
                 '--type AzureWatson -g {rg} -n {name} --storage-accounts {sa_1}',
                 checks=[
                     self.check('storageAccountIds[0]', '{sa_id_1}'),
                     self.check('name', 'AzureWatson')
                 ])

        self.cmd('monitor log-analytics workspace linked-storage list '
                 '-g {rg} -n {name}',
                 checks=[
                     self.check('length(@)', 2)
                 ])

        self.cmd('monitor log-analytics workspace linked-storage delete '
                 '--type AzureWatson -g {rg} -n {name} -y')

        self.cmd('monitor log-analytics workspace linked-storage list '
                 '-g {rg} -n {name}',
                 checks=[
                     self.check('length(@)', 1)
                 ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_public_access', location='eastus')
    def test_monitor_log_analytics_workspace_public_access(self, resource_group):
        self.kwargs.update({
            'name': self.create_random_name('clitest', 20),
            'name_2': self.create_random_name('clitest', 20)
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --tags clitest=myron "
                 "--query-access Disabled --ingestion-access Disabled",
                 checks=[
                     self.check('publicNetworkAccessForIngestion', 'Disabled'),
                     self.check('publicNetworkAccessForQuery', 'Disabled')
                 ])

        self.cmd("monitor log-analytics workspace update -g {rg} -n {name} "
                 "--query-access Disabled --ingestion-access Enabled",
                 checks=[
                     self.check('publicNetworkAccessForIngestion', 'Enabled'),
                     self.check('publicNetworkAccessForQuery', 'Disabled')
                 ])

        self.cmd("monitor log-analytics workspace show -g {rg} -n {name}", checks=[
            self.check('publicNetworkAccessForIngestion', 'Enabled'),
            self.check('publicNetworkAccessForQuery', 'Disabled')
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_recover', location='eastus')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_recover(self, resource_group):
        workspace_name = self.create_random_name('clitest', 20)
        self.kwargs.update({
            'name': workspace_name
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {name} --quota 1 --level 100 --sku CapacityReservation", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'CapacityReservation'),
            self.check('sku.capacityReservationLevel', 100),
            self.check('workspaceCapping.dailyQuotaGb', 1.0)
        ])

        self.cmd("monitor log-analytics workspace update -g {rg} -n {name} --quota 2 --level 200", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('sku.capacityReservationLevel', 200),
            self.check('workspaceCapping.dailyQuotaGb', 2.0)
        ])

        self.kwargs.update({
            'table_name': 'Syslog'
        })

        self.cmd("monitor log-analytics workspace table update -g {rg} --workspace-name {name} -n {table_name} --retention-time 30 --debug", checks=[
            self.check('retentionInDays', 30)
        ])

        self.cmd("monitor log-analytics workspace list-deleted-workspaces -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

        self.cmd("monitor log-analytics workspace delete -g {rg} -n {name} -y")

        self.cmd("monitor log-analytics workspace list-deleted-workspaces -g {rg}", checks=[
            self.check('length(@)', 1)
        ])

        self.cmd("monitor log-analytics workspace recover -g {rg} -n {name}", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'CapacityReservation'),
            self.check('sku.capacityReservationLevel', 200),
            self.check('workspaceCapping.dailyQuotaGb', 2.0)
        ])

        self.cmd("monitor log-analytics workspace show -g {rg} -n {name}", checks=[
            self.check('provisioningState', 'Succeeded'),
            self.check('retentionInDays', 30),
            self.check('sku.name', 'CapacityReservation'),
            self.check('sku.capacityReservationLevel', 200),
            self.check('workspaceCapping.dailyQuotaGb', 2.0)
        ])

        self.cmd("monitor log-analytics workspace list-deleted-workspaces -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

        self.cmd("monitor log-analytics workspace delete -g {rg} -n {name} --force -y")

        self.cmd("monitor log-analytics workspace list-deleted-workspaces -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

        self.cmd("monitor log-analytics workspace list -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_saved_search', location='eastus')
    def test_monitor_log_analytics_workspace_saved_search(self, resource_group):
        self.kwargs.update({
            'workspace_name': self.create_random_name('clitest', 20),
            'saved_search_name': 'clitest',
            'category': 'cli',
            'category_2': 'cli2',
            'query': "Heartbeat | getschema",
            'query_2': "AzureActivity | summarize count() by bin(TimeGenerated, 1h)",
            'display_name': 'myclitest',
            'display_name_2': 'myclitest2',
            'function_alias': 'myfun',
            'function_param': "a:string = value",
            'function_alias_2': 'myfun2',
            'function_param_2': "a2:string = value",
            'rg': resource_group
        })

        self.cmd("monitor log-analytics workspace create -g {rg} -n {workspace_name} --tags clitest=myron")

        # Disable checks due to service issue: https://github.com/Azure/azure-rest-api-specs/issues/12363, will enable checks after service issue is fixed.
        self.cmd('monitor log-analytics workspace saved-search create -g {rg} --workspace-name {workspace_name} -n {saved_search_name} '
                 '--category {category} --display-name {display_name} -q "{query}" --fa {function_alias} '
                 '--fp "{function_param}" --tags a=b c=d',
                 checks=[
                     self.check('category', '{category}'),
                     self.check('displayName', '{display_name}'),
                     self.check('query', "{query}"),
                     self.check('functionAlias', '{function_alias}'),
                     self.check('functionParameters', '{function_param}'),
                     self.check('length(tags)', 2)
                 ])

        self.cmd('monitor log-analytics workspace saved-search show -g {rg} --workspace-name {workspace_name} -n {saved_search_name}', checks=[
            self.check('category', '{category}'),
            self.check('displayName', '{display_name}'),
            self.check('query', "Heartbeat | getschema"),
            self.check('functionAlias', '{function_alias}'),
            self.check('functionParameters', '{function_param}'),
            self.check('length(tags)', 2)
        ])
        self.cmd('monitor log-analytics workspace saved-search list -g {rg} --workspace-name {workspace_name}', checks=[
        ])

        self.cmd(
            'monitor log-analytics workspace saved-search update -g {rg} --workspace-name {workspace_name} -n {saved_search_name} '
            '--category {category_2} --display-name {display_name_2} -q "{query_2}" --fa {function_alias_2} '
            '--fp "{function_param_2}" --tags a=c f=e',
            checks=[
                self.check('category', '{category_2}'),
                self.check('displayName', '{display_name_2}'),
                self.check('query', "{query_2}"),
                self.check('functionAlias', '{function_alias_2}'),
                self.check('functionParameters', '{function_param_2}'),
                self.check('length(tags)', 2),
                self.check('tags[0].value', 'c'),
                self.check('tags[1].value', 'e')
            ])

        self.cmd('monitor log-analytics workspace saved-search delete -g {rg} --workspace-name {workspace_name} -n {saved_search_name} -y')

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_data_export', location='eastus')
    @StorageAccountPreparer(name_prefix='saws1', kind='StorageV2', sku='Standard_LRS', parameter_name='account_1',
                            location='eastus')
    def test_monitor_log_analytics_workspace_data_export(self, resource_group, account_1):
        from azure.mgmt.core.tools import resource_id
        self.kwargs.update({
            'workspace_name': self.create_random_name('clitest', 20),
            'data_export_name': 'clitest',
            'data_export_name_2': 'clitest2',
            'sa_1': account_1,
            'sa_id_1': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=account_1,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'namespacename': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'eventhubname': "hub_name",
            'rg': resource_group
        })

        self.cmd(
            "monitor log-analytics workspace create -g {rg} -n {workspace_name} --quota 1 --level 100 --sku CapacityReservation",
            checks=[
                self.check('provisioningState', 'Succeeded'),
                self.check('retentionInDays', 30),
                self.check('sku.name', 'CapacityReservation'),
                self.check('sku.capacityReservationLevel', 100),
                self.check('workspaceCapping.dailyQuotaGb', 1.0)
            ])
        self.kwargs.update({
            'table_name': 'Syslog'
        })

        self.cmd('monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name} '
                 '--destination {sa_id_1} --enable -t {table_name}',
                 checks=[
                 ])

        from azure.core.exceptions import HttpResponseError
        with self.assertRaisesRegex(HttpResponseError, 'You are adding a destination that is already defined in rule: clitest. Destination must be unique across export rules in your workspace. See http://aka.ms/LADataExport#limitations'):
            self.cmd('monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name_2} '
                     '--destination {sa_id_1} --enable -t "SecurityEvent Heartbeat"',
                     checks=[
                     ])
        with self.assertRaisesRegex(HttpResponseError, 'You are adding a destination that is already defined in rule: clitest. Destination must be unique across export rules in your workspace. See http://aka.ms/LADataExport#limitations'):
            self.cmd('monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name_2} '
                     '--destination {sa_id_1} --enable -t {table_name}',
                     checks=[
                     ])
        with self.assertRaisesRegex(HttpResponseError, 'You are adding a destination that is already defined in rule: clitest. Destination must be unique across export rules in your workspace. See http://aka.ms/LADataExport#limitations'):
            self.cmd('monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name_2} '
                     '--destination {sa_id_1} --enable -t ABC',
                     checks=[
                     ])
        with self.assertRaisesRegex(HttpResponseError,'You are adding a destination that is already defined in rule: clitest. Destination must be unique across export rules in your workspace. See http://aka.ms/LADataExport#limitations'):
            self.cmd('monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name_2} '
                     '--destination {sa_id_1} --enable -t AppPerformanceCounters',
                     checks=[
                     ])
        self.cmd('monitor log-analytics workspace data-export show -g {rg} --workspace-name {workspace_name} -n {data_export_name}', checks=[
        ])

        self.cmd('monitor log-analytics workspace data-export list -g {rg} --workspace-name {workspace_name}', checks=[
            self.check('length(@)', 1)
        ])

        result = self.cmd('eventhubs namespace create --resource-group {rg} --name {namespacename}').get_output_in_json()
        self.kwargs.update({
            'namespace_id': result['id']
        })
        result = self.cmd('eventhubs eventhub create --resource-group {rg} --namespace-name {namespacename} --name {eventhubname}').get_output_in_json()
        self.kwargs.update({
            'eventhub_id': result['id']
        })
        self.cmd(
            'monitor log-analytics workspace data-export update -g {rg} --workspace-name {workspace_name} -n {data_export_name} '
            '--destination {namespace_id} --enable true -t Usage Alert',
            checks=[
            ])

        self.cmd('eventhubs eventhub list -g {rg} --namespace-name {namespacename}')

        self.cmd('monitor log-analytics workspace data-export delete -g {rg} --workspace-name {workspace_name} -n {data_export_name} -y')

        self.cmd(
            'monitor log-analytics workspace data-export create -g {rg} --workspace-name {workspace_name} -n {data_export_name} '
            '--destination {eventhub_id} --enable false -t {table_name}',
            checks=[
            ])

        self.cmd('monitor log-analytics workspace data-export delete -g {rg} --workspace-name {workspace_name} -n {data_export_name} -y')
        self.cmd('monitor log-analytics workspace data-export list -g {rg} --workspace-name {workspace_name}', checks=[
            self.check('length(@)', 0)
        ])

    @record_only()
    @unittest.skip('resource not available')
    def test_monitor_log_analytics_workspace_data_collection_rules(self):
        self.kwargs.update({
            'ws_name': 'wsn1',
            'rule_name': 'rule11',
            'rule_id': '/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272f09590/resourceGroups/tbtest/providers/Microsoft.Insights/dataCollectionRules/rule11'
        })
        # extension command
        # rule = self.cmd('monitor data-collection rule show -g tbtest -n rule11').get_output_in_json()
        # self.kwargs.update({
        #     'rule_id': rule['id']
        # })
        self.cmd('monitor log-analytics workspace update -g tbtest -n {ws_name} --data-collection-rule {rule_id}', checks=[
            # self.check('defaultDataCollectionRuleResourceId', '{rule_id}')
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_table', location='WestEurope')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_table(self, resource_group):

        self.kwargs.update({
            'ws_name':self.create_random_name('ws-', 10),
            'table_name': self.create_random_name('TB', 10) + '_CL',
            'table2_name': self.create_random_name('TB', 10) + '_SRCH',
            'table3_name': self.create_random_name('TB', 10) + '_RST'
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        self.cmd('monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 45 --total-retention-time 70 --plan Analytics --columns col1=guid TimeGenerated=datetime', checks=[
            self.check('name', '{table_name}'),
            self.check('retentionInDays', 45),
            self.check('totalRetentionInDays', 70),
            self.check('schema.columns[0].name', 'col1'),
            self.check('schema.columns[0].type', 'guid'),
            self.check('schema.columns[1].name', 'TimeGenerated'),
            self.check('schema.columns[1].type', 'datetime'),
        ])
        self.cmd('monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 50 --total-retention-time 80 --columns col2=guid TimeGenerated=datetime', checks=[
            self.check('name', '{table_name}'),
            self.check('retentionInDays', 50),
            self.check('totalRetentionInDays', 80),
            self.check('schema.columns[0].name', 'col2'),
            self.check('schema.columns[0].type', 'guid'),
        ])
        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}', checks=[
            self.check('name', '{table_name}'),
            self.check('retentionInDays', 50),
            self.check('totalRetentionInDays', 80),
        ])

        self.cmd('monitor log-analytics workspace table delete -g {rg} -n {table_name} --workspace-name {ws_name} -y')

        self.cmd('monitor log-analytics workspace table search-job create -n {table2_name} -g {rg} --workspace-name {ws_name} --retention-time 50 --total-retention-time 80 --start-search-time "2021-08-01 05:29:18" --end-search-time "2021-08-02 05:29:18" --search-query "Heartbeat" --limit 1', checks=[
            self.check('name', '{table2_name}'),
            self.check('searchResults.query', 'Heartbeat'),
            self.check('searchResults.limit', 1),
            self.check('searchResults.sourceTable', "Heartbeat"),
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_table_total_retention', location='WestEurope')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_table_total_retention_upper_fix(self, resource_group):
        self.kwargs.update({
            'ws_name': self.create_random_name('ws-', 10),
            'table_name': self.create_random_name('TB', 10) + '_CL',
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        self.cmd(
            'monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 45 --total-retention-time 2556 --plan Analytics --columns col1=guid TimeGenerated=datetime',
            checks=[
                self.check('name', '{table_name}'),
                self.check('retentionInDays', 45),
                self.check('totalRetentionInDays', 2556),
                self.check('schema.columns[0].name', 'col1'),
                self.check('schema.columns[0].type', 'guid'),
                self.check('schema.columns[1].name', 'TimeGenerated'),
                self.check('schema.columns[1].type', 'datetime'),
            ])
        self.cmd(
            'monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --total-retention-time 400 --columns col2=guid TimeGenerated=datetime',
            checks=[
                self.check('name', '{table_name}'),
                self.check('totalRetentionInDays', 400),
                self.check('schema.columns[0].name', 'col2'),
                self.check('schema.columns[0].type', 'guid'),
            ])
        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}',
                 checks=[
                     self.check('name', '{table_name}'),
                     self.check('retentionInDays', 45),
                     self.check('totalRetentionInDays', 400),
                 ])

        self.cmd(
            'monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --total-retention-time 2556 --retention-time 50',
            checks=[
                self.check('name', '{table_name}'),
                self.check('retentionInDays', 50),
                self.check('totalRetentionInDays', 2556),
            ])
        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}',
                 checks=[
                     self.check('name', '{table_name}'),
                     self.check('retentionInDays', 50),
                     self.check('totalRetentionInDays', 2556),
                 ])

        self.cmd('monitor log-analytics workspace table delete -g {rg} -n {table_name} --workspace-name {ws_name} -y')


    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_table', location='WestEurope')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_table_restore_datetime_adjust(self, resource_group):

        self.kwargs.update({
            'ws_name':self.create_random_name('ws-', 10),
            'table_name': self.create_random_name('TB', 10) + '_CL',
            'table2_name': self.create_random_name('TB', 10) + '_RST'
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        self.cmd('monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 45 --total-retention-time 70 --plan Analytics --columns col1=guid TimeGenerated=datetime', checks=[
            self.check('name', '{table_name}'),
            self.check('retentionInDays', 45),
            self.check('totalRetentionInDays', 70),
            self.check('schema.columns[0].name', 'col1'),
            self.check('schema.columns[0].type', 'guid'),
            self.check('schema.columns[1].name', 'TimeGenerated'),
            self.check('schema.columns[1].type', 'datetime'),
        ])

        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}', checks=[
            self.check('name', '{table_name}')
        ])

        import re
        pattern = re.compile("Table {0} has no data on the specified date range".format(self.kwargs["table_name"]) + r"(.*?)8/11/2023 12:00:00 AM(.*?)8/17/2023 12:00:00 AM(.*?)")
        with self.assertRaisesRegex(HttpResponseError, pattern):
            self.cmd('monitor log-analytics workspace table restore create -n {table2_name} -g {rg} --workspace-name {ws_name} --restore-source-table {table_name} --start-restore-time "2023-08-11 05:29:18" --end-restore-time "2023-08-17 05:29:18"')

        self.cmd('monitor log-analytics workspace table delete -g {rg} -n {table_name} --workspace-name {ws_name} -y')


    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_table_name_valid', location='WestEurope')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_table_name_validation(self, resource_group):

        self.kwargs.update({
            'ws_name':self.create_random_name('ws-', 10),
            'table_name': self.create_random_name('TB', 10) + '_CL',
            'table1_name': self.create_random_name('TB', 10),
            'table2_name': self.create_random_name('TB', 10)
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        self.cmd('monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 45 --total-retention-time 70 --plan Analytics --columns col1=guid TimeGenerated=datetime', checks=[
            self.check('name', '{table_name}'),
            self.check('retentionInDays', 45),
            self.check('totalRetentionInDays', 70),
            self.check('schema.columns[0].name', 'col1'),
            self.check('schema.columns[0].type', 'guid'),
            self.check('schema.columns[1].name', 'TimeGenerated'),
            self.check('schema.columns[1].type', 'datetime'),
        ])

        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}', checks=[
            self.check('name', '{table_name}')
        ])
        from knack.util import CLIError
        with self.assertRaisesRegex(CLIError, "usage: The table name needs to end with _RST"):
            self.cmd('monitor log-analytics workspace table restore create -n {table2_name} -g {rg} --workspace-name {ws_name} --restore-source-table {table_name} --start-restore-time "2023-08-11 05:29:18" --end-restore-time "2023-08-17 05:29:18"')

        self.cmd('monitor log-analytics workspace table delete -g {rg} -n {table_name} --workspace-name {ws_name} -y')

        with self.assertRaisesRegex(CLIError, "usage: The table name needs to end with _SRCH"):
            self.cmd('monitor log-analytics workspace table search-job create -n {table1_name} -g {rg} --workspace-name {ws_name} --retention-time 50 --total-retention-time 80 --start-search-time "2023-08-21 05:29:18" --end-search-time "2023-09-12 05:29:18" --search-query "Heartbeat" --limit 1')

        from azure.cli.core.azclierror import InvalidArgumentValueError
        with self.assertRaisesRegex(InvalidArgumentValueError, "usage: The table name needs to end with _SRCH"):
            self.cmd('monitor log-analytics workspace table search-job cancel -n {table1_name} -g {rg} --workspace-name {ws_name}')


    @ResourceGroupPreparer(name_prefix='cli_test_monitor_workspace_table_retention_validation', location='WestEurope')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_table_retention_validation(self, resource_group):
        self.kwargs.update({
            'ws_name': self.create_random_name('ws-', 10),
            'table_name': self.create_random_name('TB', 10) + '_CL',
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        from azure.cli.core.azclierror import InvalidArgumentValueError
        with self.assertRaisesRegex(InvalidArgumentValueError, "usage error: --retention-time should between 4 and 730"):
            self.cmd('monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 3 --total-retention-time 3 --plan Analytics --columns col1=guid TimeGenerated=datetime')

        with self.assertRaisesRegex(InvalidArgumentValueError, "usage error: --total-retention-time should between 4 and 4383"):
            self.cmd('monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time -1 --total-retention-time 3 --plan Analytics --columns col1=guid TimeGenerated=datetime')

        self.cmd(
            'monitor log-analytics workspace table create -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time -1 --total-retention-time -1 --plan Analytics --columns col1=guid TimeGenerated=datetime',
            checks=[
                self.check('name', '{table_name}'),
                self.check('schema.columns[0].name', 'col1'),
                self.check('schema.columns[0].type', 'guid'),
                self.check('schema.columns[1].name', 'TimeGenerated'),
                self.check('schema.columns[1].type', 'datetime'),
            ])
        with self.assertRaisesRegex(InvalidArgumentValueError, "usage error: --retention-time should between 4 and 730"):
            self.cmd('monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 2 --total-retention-time 2')
        with self.assertRaisesRegex(InvalidArgumentValueError, "usage error: --total-retention-time should between 4 and 4383"):
            self.cmd('monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 4 --total-retention-time 2')
        self.cmd(
            'monitor log-analytics workspace table update -g {rg} -n {table_name} --workspace-name {ws_name} --retention-time 4 --total-retention-time 4383 --columns col2=guid TimeGenerated=datetime',
            checks=[
                self.check('name', '{table_name}'),
                self.check('retentionInDays', 4),
                self.check('totalRetentionInDays', 4383),
                self.check('schema.columns[0].name', 'col2'),
                self.check('schema.columns[0].type', 'guid'),
            ])
        self.cmd('monitor log-analytics workspace table show -g {rg} -n {table_name} --workspace-name {ws_name}',
                 checks=[
                     self.check('name', '{table_name}'),
                     self.check('retentionInDays', 4),
                     self.check('totalRetentionInDays', 4383),
                 ])

        self.cmd('monitor log-analytics workspace table delete -g {rg} -n {table_name} --workspace-name {ws_name} -y')

    @ResourceGroupPreparer(name_prefix='test_monitor_log_analytics_workspace_link_target', location='WestUs')
    @AllowLargeResponse()
    def test_monitor_log_analytics_workspace_link_target_available_service(self, resource_group):
        self.kwargs.update({
            'ws_name': self.create_random_name('ws-', 10),
        })

        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws_name}')
        from azure.core.exceptions import ResourceNotFoundError
        with self.assertRaises(ResourceNotFoundError):
            self.cmd("monitor log-analytics workspace list-link-target")
        self.cmd("monitor log-analytics workspace list-available-service-tier -g {rg} --workspace-name {ws_name}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, JMESPathCheckExists,
                               JMESPathCheckNotExists)

from azure.cli.command_modules.containerapp.tests.latest.common import TEST_LOCATION
from .utils import prepare_containerapp_env_for_app_e2e_tests

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppJobsRegistryOperationsTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @ResourceGroupPreparer(location="northeurope")
    @AllowLargeResponse()
    def test_containerappjob_registry_crud(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        job = self.create_random_name(prefix='app', length=24)
        acr = self.create_random_name(prefix='acr', length=24)
        env = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(
            "az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type manual --parallelism 1 --replica-completion-count 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(
                resource_group, job, env))
        acr_server = f'{acr}.azurecr.io'
        self.cmd(f'acr create -g {resource_group} -n {acr} --sku basic --admin-enabled')
        # self.cmd(f'acr credential renew -n {acr} ')
        self.cmd(f'containerapp job registry set --server {acr}.azurecr.io -g {resource_group} -n {job}', checks=[
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].server', acr_server),
            JMESPathCheck('[0].passwordSecretRef', f'{acr}azurecrio-{acr}'),
            JMESPathCheck('[0].username', acr)
        ])
        registry_list = self.cmd(f'containerapp job registry list -g {resource_group} -n {job}', checks=[
            JMESPathCheck('length(@)', 1),
        ]).get_output_in_json()

        self.cmd(f'containerapp job registry show -g {resource_group} -n {job} --server {registry_list[0]["server"]}',
                 checks=[
                     JMESPathCheck('server', acr_server),
                     JMESPathCheck('passwordSecretRef', f'{acr}azurecrio-{acr}'),
                     JMESPathCheck('username', acr)
                 ])

        self.cmd(
            f'containerapp job registry set --server {acr}.azurecr.io -g {resource_group} -n {job} --identity system', checks=[
                JMESPathCheck('length(@)', 1),
                JMESPathCheck('[0].server', acr_server),
                JMESPathCheck('[0].identity', "system")
            ])

        self.cmd(f'containerapp job registry remove -g {resource_group} -n {job} --server {registry_list[0]["server"]}', expect_failure=False)
        self.cmd(f'containerapp job registry list -g {resource_group} -n {job}', checks=[
            JMESPathCheck('length(@)', 0),
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import azure.cli.command_modules
import re
import shutil
import py_compile
import logging

_LOGGER = logging.getLogger(__name__)


class CompactorCtx:

    def __init__(self):
        self._code_pieces = {}
        self._namespace = None
        self._write_mode = False
        self._written_namespaces = set()
        self._parent_namespaces = set()

    def add_code_piece(self, key, code):
        assert self._namespace is not None and self._write_mode, "Namespace is not set or readonly"
        if key not in self._code_pieces:
            self._code_pieces[key] = {
                self._namespace: {
                    "max_count_code": code,
                    "codes": {
                        code: 1
                    }
                }
            }
            return

        if self._namespace in self._code_pieces[key] and code in self._code_pieces[key][self._namespace]["codes"]:
            # if the code defined in the current namespace increase the count and update the max_count_code
            codes = self._code_pieces[key][self._namespace]["codes"]
            codes[code] += 1
            max_count_code = self._code_pieces[key][self._namespace]["max_count_code"]
            if codes[code] > codes[max_count_code]:
                self._code_pieces[key][self._namespace]["max_count_code"] = code
            return

        # the code is defined in other _helpers
        if self.find_code_piece_in_parent(key, code):
            return

        # add code piece
        if self._namespace not in self._code_pieces[key]:
            self._code_pieces[key] = {
                self._namespace: {
                    "max_count_code": code,
                    "codes": {
                        code: 1
                    }
                }
            }
            return

        self._code_pieces[key][self._namespace]["codes"][code] = 1

    def set_current_namespace(self, namespace, write_mode):
        assert not write_mode or namespace not in self._written_namespaces
        self._namespace = namespace
        self._write_mode = write_mode
        if not namespace:
            # clean namespace
            return

        self._parent_namespaces.clear()
        pieces = namespace.split(os.sep)
        for i in range(1, len(pieces)-1):
            parent_namespace = os.sep.join(pieces[:-i])
            self._parent_namespaces.add(parent_namespace)

        if write_mode:
            self._written_namespaces.add(namespace)

    def find_code_piece_in_parent(self, key, code):
        """find code piece in frozen namespaces"""
        if key not in self._code_pieces:
            return
        for namespace, value in self._code_pieces[key].items():
            if namespace not in self._parent_namespaces:
                continue
            if value["max_count_code"] == code and value["codes"][code] > 1:
                return namespace

    def fetch_code_piece(self, key, code):
        assert not self._write_mode, "Fetch code piece is not supported in write mode"
        parent_namespace = self.find_code_piece_in_parent(key, code)
        if parent_namespace:
            return parent_namespace
        if key in self._code_pieces and self._namespace in self._code_pieces[key] and \
                code == self._code_pieces[key][self._namespace]["max_count_code"] and \
                self._code_pieces[key][self._namespace]["codes"][code] > 1:
            return self._namespace

    def fetch_helper_code_piece(self, key):
        assert not self._write_mode, "Fetch code piece is not supported in write mode"
        if key not in self._code_pieces or self._namespace not in self._code_pieces[key]:
            return
        value = self._code_pieces[key][self._namespace]
        code = value["max_count_code"]
        if value["codes"][code] > 1:
            return code


_PY_HEADER = """# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------
# pylint: skip-file
# flake8: noqa

"""


class MainModuleCompactor:

    _command_group_pattern = re.compile(r'^class\s+(.*)\(.*AAZCommandGroup.*\)\s*:\s*$')
    _command_pattern = re.compile(r'^class\s+(.*)\(.*AAZ(Wait)?Command.*\)\s*:\s*$')
    _command_helper_pattern = re.compile(r'^class\s+(_(.+)Helper)\s*:\s*$')
    _command_helper_func_pattern = re.compile(r'^\s{4}def (.+)\(cls.*\)\s*:\s*$')
    _file_end_pattern = re.compile(r'^__all__ = \[.*]\s*$')
    _class_method_register = "    @classmethod"

    def __init__(self, mod_name, compiled_to_pyc=True):
        self._mod_name = mod_name
        self._modules_dir = azure.cli.command_modules.__path__[0]
        self._folder = self._get_module_folder()
        self._compiled_to_pyc = compiled_to_pyc

    def compact(self):
        _LOGGER.info("Compacting {} folder:".format(self._get_aaz_folder()))
        self._create_compact_aaz_folder()
        from azure.cli.core.profiles import AZURE_API_PROFILES
        from azure.cli.core.aaz.utils import get_aaz_profile_module_name
        for profile in AZURE_API_PROFILES:
            _LOGGER.info("Compacting profile {}".format(profile))

            profile_mod_name = get_aaz_profile_module_name(profile)
            profile_path = self._get_aaz_rg_path(profile_mod_name)
            if not os.path.exists(profile_path):
                continue

            # create profile folders
            compact_folder = self._get_compact_aaz_rg_path(profile_mod_name)
            os.mkdir(compact_folder)
            self._write_py_file(os.path.join(compact_folder, '__init__.py'), content="")

            ctx = CompactorCtx()
            self.compact_sub_resource_groups(ctx, profile_mod_name)

    def replace(self):
        aaz_folder = self._get_aaz_folder()
        compact_aaz_folder = self._get_compact_aaz_folder()
        _LOGGER.info("Removing {} folder".format(aaz_folder))
        shutil.rmtree(aaz_folder)
        _LOGGER.info("Move {} folder to {}".format(compact_aaz_folder, aaz_folder))
        shutil.move(compact_aaz_folder, aaz_folder)

    def _write_py_file(self, path, content):
        _LOGGER.debug("Writing python file {}".format(path))
        init_content = _PY_HEADER + content
        with open(path, 'w') as f:
            f.write(init_content)
        if self._compiled_to_pyc:
            py_compile.compile(path)

    def _create_compact_aaz_folder(self):
        folder = self._get_compact_aaz_folder()
        if os.path.exists(folder):
            shutil.rmtree(folder)
        os.mkdir(folder)
        self._write_py_file(os.path.join(folder, '__init__.py'), content="")

    def compact_sub_resource_groups(self, ctx: CompactorCtx, dirs):
        folder = self._get_aaz_rg_path(dirs)
        assert os.path.isdir(folder), f'Invalid folder path {folder}'
        for name in os.listdir(folder):
            sub_dir = os.path.join(dirs, name)
            self.compact_resource_group(ctx, sub_dir)

    def compact_resource_group(self, ctx: CompactorCtx, dirs):
        folder = self._get_aaz_rg_path(dirs)
        init_file = os.path.join(folder, '__init__.py')
        if not os.path.exists(init_file):
            return
        if not os.path.isfile(init_file):
            raise ValueError("Invalid init file: {}".format(init_file))

        compact_folder = self._get_compact_aaz_rg_path(dirs)
        os.mkdir(compact_folder)
        self.compact_resource_group_commands(ctx, dirs)
        self.compact_sub_resource_groups(ctx, dirs)

    def compact_resource_group_commands(self, ctx: CompactorCtx, dirs):
        folder = self._get_aaz_rg_path(dirs)
        cmds_file = os.path.join(folder, '__cmds.py')
        if os.path.exists(cmds_file):
            raise ValueError("Module is already compacted: {}".format(cmds_file))

        ctx.set_current_namespace(dirs, write_mode=True)

        compact_folder = self._get_compact_aaz_rg_path(dirs)

        _LOGGER.debug("Parsing folder {}".format(folder))
        cmds_content, grp_cls = self._parse_cmd_group_file(folder, None)

        cmds_content, cmd_clses, cmd_cls_helpers = self._parse_cmd_files(folder, cmds_content)

        if cmd_cls_helpers:
            link_helper_codes = {}
            for helper in cmd_cls_helpers.values():
                helper_codes = helper["codes"]
                for k, v in helper_codes.items():
                    ctx.add_code_piece(k, v)

            helper_content = ""

            ctx.set_current_namespace(dirs, write_mode=False)
            for name, helper in cmd_cls_helpers.items():
                helper_codes = helper["codes"]
                helper_properties = helper["properties"]
                helper_cls_links = []
                helper_cls_content = f'class {name}:\n{helper_properties}'
                for k, v in helper_codes.items():
                    link_dirs = ctx.fetch_code_piece(k, v)
                    if not link_dirs:
                        helper_cls_content += '\n'
                        helper_cls_content += v
                        continue
                    assert dirs.startswith(link_dirs)
                    if link_dirs == dirs:
                        helper_cls_links.append(f'    ("{k}", _Helper),')
                        linker_code = ctx.fetch_helper_code_piece(k)
                        assert linker_code
                        link_helper_codes[k] = linker_code
                    else:
                        relative_path = '.' * (len(dirs.split(os.sep))-len(link_dirs.split(os.sep))) + '.__cmds'
                        helper_cls_links.append(f'    ("{k}", "{relative_path}"),')
                if helper_cls_links:
                    helper_cls_content = '\n'.join(['@link_helper(','    __package__,', *helper_cls_links, ')']) + '\n' + helper_cls_content
                helper_content += '\n\n' + helper_cls_content

            if link_helper_codes:
                cmds_content += f'\n\nclass _Helper:\n'
                for v in link_helper_codes.values():
                    cmds_content += "\n"
                    assert v.startswith(self._class_method_register)
                    cmds_content += f"    @staticmethod" + v[len(self._class_method_register):]

            cmds_content += helper_content

        init_content = "from .__cmds import *\n" if cmds_content else ""
        self._write_py_file(os.path.join(compact_folder, '__init__.py'), content=init_content)

        if cmds_content:
            if cmd_clses:
                all_clses = [f'"{cmd_cls}"' for cmd_cls in cmd_clses]
            else:
                all_clses = []

            if grp_cls:
                all_clses.append(f'"{grp_cls}"')

            if all_clses:
                cmds_content += ''.join(['\n', '\n', f'__all__ = [{",".join(all_clses)}]\n'])

            self._write_py_file(os.path.join(compact_folder, '__cmds.py'), content=cmds_content)

        ctx.set_current_namespace(None, write_mode=False)

    def _parse_cmd_group_file(self, folder, cmds_content):
        cmd_group_file = os.path.join(folder, '__cmd_group.py')
        if not os.path.isfile(cmd_group_file):
            return None, None

        _LOGGER.debug("Parsing command group file {}".format(cmd_group_file))

        grp_cls = None

        if not cmds_content:
            cmds_content = "from azure.cli.core.aaz import *\n"

        cg_lines = []
        with open(cmd_group_file, 'r') as f:
            while f.readable():
                line = f.readline()
                if not cg_lines and line.startswith('@register_command_group('):
                    cg_lines.append(line)
                    continue
                if not grp_cls:
                    match = self._command_group_pattern.match(line)
                    if match:
                        grp_cls = match[1]
                        cg_lines.append(line)
                        continue
                if not cg_lines:
                    continue

                if self._file_end_pattern.match(line):
                    break
                cg_lines.append(line)
        while cg_lines and not cg_lines[-1].strip():
            cg_lines.pop()

        cmds_content += ''.join(['\n', '\n', *cg_lines])

        _LOGGER.debug("Parsed CommandGroup Class: {}".format(grp_cls))

        return cmds_content, grp_cls

    def _parse_cmd_files(self, folder,  cmds_content):
        cmd_clses = []
        cmd_cls_helpers = {}
        for name in os.listdir(folder):
            if name.startswith('__') or not name.startswith('_') or not name.endswith('.py'):
                continue
            cmd_file = os.path.join(folder, name)
            if not os.path.isfile(cmd_file):
                continue
            cmd_cls, cmd_lines, cmd_helper_lines = self._parse_cmd_file(cmd_file)
            if not cmd_cls:
                continue
            cmd_clses.append(cmd_cls)
            cmds_content += ''.join(['\n', '\n', *cmd_lines])

            if cmd_helper_lines:
                helper_name, helper_properties, helper_codes = self._parse_cmd_helper_lines(cmd_helper_lines)
                if helper_properties or helper_codes:
                    cmd_cls_helpers[helper_name] = {
                        "properties": helper_properties,
                        "codes": helper_codes
                    }

        _LOGGER.debug("Parsed Command Classes: {}".format(cmd_clses))

        return cmds_content, cmd_clses, cmd_cls_helpers

    def _parse_cmd_file(self, cmd_file):
        _LOGGER.debug("Parsing command file {}".format(cmd_file))

        cmd_cls = None
        cmd_lines = []
        cmd_helper_lines = []
        with open(cmd_file, 'r') as f:
            # read the cmd_cls definition
            while f.readable():
                line = f.readline()
                if not cmd_lines and line.startswith('@register_command('):
                    cmd_lines.append(line)
                    continue
                if not cmd_cls:
                    match = self._command_pattern.match(line)
                    if match:
                        cmd_cls = match[1]
                        cmd_lines.append(line)
                        continue
                if not cmd_lines:
                    continue

                if self._file_end_pattern.match(line):
                    break
                match = self._command_helper_pattern.match(line)
                if match:
                    cmd_helper_lines.append(line)
                    break
                cmd_lines.append(line)

            # read the cmd_helper_cls definition
            while f.readable() and cmd_helper_lines:
                line = f.readline()
                if self._file_end_pattern.match(line):
                    break
                cmd_helper_lines.append(line)

        while cmd_lines and not cmd_lines[-1].strip():
            cmd_lines.pop()

        while cmd_helper_lines and not cmd_helper_lines[-1].strip():
            cmd_helper_lines.pop()

        return cmd_cls, cmd_lines, cmd_helper_lines

    def _parse_cmd_helper_lines(self, cmd_helper_lines):
        helper_name, helper_codes = None, {}
        idx = 0
        code_key = None
        code_lines = []
        properties_lines = []
        while idx < len(cmd_helper_lines):
            line = cmd_helper_lines[idx]
            if not helper_name:
                match = self._command_helper_pattern.match(line)
                if match:
                    helper_name = match[1]
                    continue
            if not helper_name:
                continue
            if line.startswith(self._class_method_register) or line.startswith("    _"):
                if code_key and code_lines:
                    while code_lines and not code_lines[-1].strip():
                        code_lines.pop()
                    helper_codes[code_key] = ''.join(code_lines)
                code_lines = []
                code_key = None
                if line.startswith("    _"):
                    while not line.startswith(self._class_method_register) and idx < len(cmd_helper_lines):
                        properties_lines.append(line)
                        idx += 1
                        line = cmd_helper_lines[idx]

            if line.startswith(self._class_method_register):
                code_lines.append(line)
                idx += 1
                line = cmd_helper_lines[idx]
                match = self._command_helper_func_pattern.match(line)
                if match:
                    code_key = match[1]

            if code_key:
                code_lines.append(line)

            idx += 1

        if code_key and code_lines:
            while code_lines and not code_lines[-1].strip():
                code_lines.pop()
            helper_codes[code_key] = ''.join(code_lines)

        properties_code = ''.join(properties_lines)

        return helper_name, properties_code, helper_codes

    def _get_module_folder(self):
        module_folder = os.path.join(self._modules_dir, self._mod_name.replace('-', '_').lower())
        if not os.path.exists(module_folder):
            raise ValueError("Module folder is not exist: {}".format(module_folder))
        return module_folder

    def _get_aaz_folder(self):
        return os.path.join(self._folder, 'aaz')

    def _get_aaz_rg_path(self, dirs):
        return os.path.join(self._get_aaz_folder(), dirs)

    def _get_compact_aaz_folder(self):
        return os.path.join(self._folder, 'aaz_compact')

    def _get_compact_aaz_rg_path(self, dirs):
        return os.path.join(self._get_compact_aaz_folder(), dirs)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    for module in ["network", "vm"]:
        compactor = MainModuleCompactor(module)
        compactor.compact()
        compactor.replace()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer

# POOL_DEFAULT = "--service-level 'Premium' --size 4398046511104"
POOL_DEFAULT = "--service-level 'Premium' --size 4"
VOLUME_DEFAULT = "--service-level 'Premium' --usage-threshold 107374182400"
RG_LOCATION = "eastus"
# No tidy up of tests required. The resource group is automatically removed


class AzureNetAppFilesBackupvaultServiceScenarioTest(ScenarioTest):
    def setup_vnet(self, rg, vnet_name, subnet_name, ip_pre):
        self.cmd("az network vnet create -n %s --resource-group %s -l %s --address-prefix %s/16" % (vnet_name, rg, RG_LOCATION, ip_pre))
        self.cmd("az network vnet subnet create -n %s -g %s --vnet-name %s --address-prefixes '%s/24' --delegations 'Microsoft.Netapp/volumes'" % (subnet_name, rg, vnet_name, ip_pre))

    def current_subscription(self):
        subs = self.cmd("az account show").get_output_in_json()
        return subs['id']

    def create_volume(self, account_name, pool_name, volume_name1, rg, tags=None, volume_name2=None):
        vnet_name = self.create_random_name(prefix='cli-vnet-', length=24)
        creation_token = volume_name1
        subnet_name = self.create_random_name(prefix='cli-subnet-', length=16)
        subnet_id = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s/subnets/%s" % (self.current_subscription(), rg, vnet_name, subnet_name)
        tag = "--tags '%s'" % tags if tags is not None else ""

        self.setup_vnet(rg, vnet_name, subnet_name, '10.12.0.0')
        self.cmd("az netappfiles account create -g %s -a '%s' -l %s" % (rg, account_name,RG_LOCATION)).get_output_in_json()
        self.cmd("az netappfiles pool create -g %s -a %s -p %s -l %s %s %s" % (rg, account_name, pool_name,RG_LOCATION, POOL_DEFAULT, tag)).get_output_in_json()
        volume1 = self.cmd("az netappfiles volume create --resource-group %s --account-name %s --pool-name %s --volume-name %s -l %s %s --creation-token %s --vnet %s --subnet %s %s" % (rg, account_name, pool_name, volume_name1, RG_LOCATION, VOLUME_DEFAULT, creation_token,vnet_name, subnet_id, tag)).get_output_in_json()

        if volume_name2:
            creation_token = volume_name2
            self.cmd("az netappfiles volume create -g %s -a %s -p %s -v %s -l %s %s --creation-token %s --subnet-id %s --tags '%s' --vnet %s" % (rg, account_name, pool_name, volume_name2, RG_LOCATION, VOLUME_DEFAULT, creation_token, subnet_id, tags, vnet_name)).get_output_in_json()

        return volume1

    @ResourceGroupPreparer(name_prefix='cli_tests_rg', additional_tags={'owner': 'cli_test'})
    def test_create_delete_backupvault(self,resource_group):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        vault_name = self.create_random_name(prefix='cli-backupvault-', length=24)
        tags = "Tag1=Value1 Tag2=Value2"
        self.kwargs.update({
            'account_name': account_name,
            'vault_name': vault_name
        })
        self.cmd(f"az netappfiles account create -g {resource_group} -a {account_name} -l {RG_LOCATION}")

        backupvault = self.cmd(f"az netappfiles account backup-vault create -g {resource_group} -a {account_name} -n {vault_name} -l {RG_LOCATION} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name
        #assert volume['tags']['Tag1'] == 'Value1'
        #assert volume['tags']['Tag2'] == 'Value2'

        backupvault_list = self.cmd(f"az netappfiles account backup-vault list --resource-group {resource_group} -a {account_name}").get_output_in_json()
        assert len(backupvault_list) == 1

        self.cmd(f"az netappfiles account backup-vault delete --resource-group {resource_group} -a {account_name} -n {vault_name} -y")
        backupvault_list = self.cmd(f"az netappfiles account backup-vault list --resource-group {resource_group} -a {account_name}").get_output_in_json()
        assert len(backupvault_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_tests_rg', additional_tags={'owner': 'cli_test'})
    def test_get_backupvault_by_name(self, resource_group):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        vault_name = self.create_random_name(prefix='cli-backupvault-', length=24)
        self.kwargs.update({
            'account_name': account_name,
            'vault_name': vault_name
        })
        tags = "Tag1=Value1 Tag2=Value2"
        self.cmd(f"az netappfiles account create -g {resource_group} -a {account_name} -l {RG_LOCATION}")

        backupvault = self.cmd(f"az netappfiles account backup-vault create -g {resource_group} -a {account_name} -n {vault_name} -l {RG_LOCATION} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name

        backupvault = self.cmd(f"az netappfiles account backup-vault show --resource-group {resource_group} -a {account_name} -n {vault_name}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name
        assert backupvault['tags']['Tag2'] == 'Value2'

        backupvault_from_id = self.cmd(f"az netappfiles account backup-vault show --ids {backupvault['id']} ").get_output_in_json()
        assert backupvault_from_id['name'] == account_name + '/' + vault_name

    @ResourceGroupPreparer(name_prefix='cli_tests_rg',additional_tags={'owner': 'cli_test'})
    def test_list_backupvault(self, resource_group):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        vault_name = self.create_random_name(prefix='cli-backupvault-', length=24)
        vault_name2 = self.create_random_name(prefix='cli-backupvault-', length=24)
        self.kwargs.update({
            'account_name': account_name,
            'vault_name': vault_name,
            'vault_name2': vault_name2
        })
        tags = "Tag1=Value1 Tag2=Value2"
        self.cmd(f"az netappfiles account create -g {resource_group} -a {account_name} -l {RG_LOCATION}")

        backupvault = self.cmd(f"az netappfiles account backup-vault create -g {resource_group} -a {account_name} -n {vault_name} -l {RG_LOCATION} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name

        backupvault = self.cmd(f"az netappfiles account backup-vault show --resource-group {resource_group} -a {account_name} -n {vault_name}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name
        assert backupvault['tags']['Tag2'] == 'Value2'

        backupvaults = self.cmd(f"az netappfiles account backup-vault list --resource-group {resource_group} -a {account_name} ").get_output_in_json()
        assert len(backupvaults) == 1

        backupvault = self.cmd(f"az netappfiles account backup-vault create -g {resource_group} -a {account_name} -n {vault_name2} -l {RG_LOCATION} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name2
        backupvaults = self.cmd(f"az netappfiles account backup-vault list --resource-group {resource_group} -a {account_name} ").get_output_in_json()
        assert len(backupvaults) == 2


    @ResourceGroupPreparer(name_prefix='cli_tests_rg', additional_tags={'owner': 'cli_test'})
    def test_update_backupvault(self, resource_group):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        vault_name = self.create_random_name(prefix='cli-backupvault-', length=24)
        self.kwargs.update({
            'account_name': account_name,
            'vault_name': vault_name
        })
        tags = "Tag1=Value1 Tag2=Value2"
        self.cmd(f"az netappfiles account create -g {resource_group} -a {account_name} -l {RG_LOCATION}")

        backupvault = self.cmd(f"az netappfiles account backup-vault create -g {resource_group} -a {account_name} -n {vault_name} -l {RG_LOCATION} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name

        tags = "Tag1=Value1 Tag2=Value2Updated"
        backupvault = self.cmd(f"az netappfiles account backup-vault update -g {resource_group} -a {account_name} -n {vault_name} --tags {tags}").get_output_in_json()
        assert backupvault['name'] == account_name + '/' + vault_name
        assert backupvault['tags']['Tag2'] == 'Value2Updated'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, record_only, ResourceGroupPreparer


class TestLogAnalyticsQueryPackScenarios(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='test_log_analytics_query_pack_crud', location="eastus2")
    def test_log_analytics_query_pack_crud(self, resource_group):
        self.kwargs.update({
            'rg': resource_group,
            'name': self.create_random_name('clitest', 20),
        })

        response = self.cmd('monitor log-analytics query-pack create -g {rg} -n {name} -l eastus2', checks=[
            self.check('name', "{name}"),
        ]).get_output_in_json()
        self.kwargs['id'] = response['id']

        self.cmd('monitor log-analytics query-pack show -g {rg} -n {name}', checks=[
            self.check('name', "{name}")
        ])

        self.cmd('monitor log-analytics query-pack show --ids {id}', checks=[
            self.check('name', "{name}")
        ])

        self.cmd("monitor log-analytics query-pack update -g {rg} -n {name} --tags flag1=v1", checks=[
            self.check('tags.flag1', 'v1')
        ])

        self.cmd("monitor log-analytics query-pack list", checks=[
            self.check('length(@)', 1)
        ])

        self.cmd("monitor log-analytics query-pack list -g {rg}", checks=[
            self.check('length(@)', 1)
        ])

        self.cmd("monitor log-analytics query-pack delete -g {rg} -n {name} --yes")

        self.cmd("monitor log-analytics query-pack list -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

    @ResourceGroupPreparer(name_prefix='test_log_analytics_query_pack_query_crud', location="eastus2")
    def test_log_analytics_query_pack_query_crud(self, resource_group):
        self.kwargs.update({
            'rg': resource_group,
            'pack_name': self.create_random_name('clitest', 20),
            'query_id': '5865fd70-3d2c-4c83-9423-7d204c3b3f90'
        })

        response = self.cmd('monitor log-analytics query-pack create -g {rg} -n {pack_name} -l eastus2', checks=[
            self.check('name', "{pack_name}"),
        ]).get_output_in_json()
        self.kwargs['pack_id'] = response['id']

        response = self.cmd('monitor log-analytics query-pack query create --query-id {query_id} -g {rg} '
                            '--query-pack-name {pack_name} --display-name query-test '
                            '--body "let newExceptionsTimeRange = 1d;let timeRangeToCheckBefore = 7d;exceptions | where timestamp < ago(timeRangeToCheckBefore) | summarize count() by problemId | join kind= rightanti (exceptions | where timestamp >= ago(newExceptionsTimeRange) | extend stack = tostring(details[0].rawStack) | summarize count(), dcount(user_AuthenticatedId), min(timestamp), max(timestamp), any(stack) by problemId) on problemId | order by  count_ desc" '
                            '--description "this is for test" '
                            '--categories "[network,monitor]" '
                            '--resource-types "[microsoft.network/loadbalancers,microsoft.insights/autoscalesettings]" '
                            '--solutions "[networkmonitoring]" '
                            '--tags "{{version:[v2022-01-01,v2021-12-01]}}"', checks=[
            self.check('properties.id', '{query_id}'),
            self.check('properties.related.categories', ['network', 'monitor']),
            self.check('properties.related.resourceTypes', ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('properties.related.solutions', ['networkmonitoring']),
            self.check('properties.tags', {"version": ["v2022-01-01", "v2021-12-01"]}),
        ]).get_output_in_json()
        self.kwargs['id'] = response['id']

        self.cmd('monitor log-analytics query-pack query show --ids {id}', checks=[self.check('properties.id', '{query_id}'),
            self.check('properties.related.categories', ['network', 'monitor']),
            self.check('properties.related.resourceTypes', ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('properties.related.solutions', ['networkmonitoring']),
            self.check('properties.tags', {"version": ["v2022-01-01", "v2021-12-01"]}),
        ])

        self.cmd('monitor log-analytics query-pack query show --query-id {query_id} -g {rg} --query-pack-name {pack_name}', checks=[self.check('properties.id', '{query_id}'),
            self.check('properties.related.categories', ['network', 'monitor']),
            self.check('properties.related.resourceTypes', ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('properties.related.solutions', ['networkmonitoring']),
            self.check('properties.tags', {"version": ["v2022-01-01", "v2021-12-01"]}),
        ])

        self.cmd(
            'monitor log-analytics query-pack query update --query-id {query_id} -g {rg} --body "heartbeat | take 10" --query-pack-name {pack_name} --categories [2]=databases --tags version[0]=null',
            checks=[self.check('properties.id', '{query_id}'),
                    self.check('properties.related.categories', ['network', 'monitor', 'databases']),
                    self.check('properties.related.resourceTypes',
                               ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
                    self.check('properties.related.solutions', ['networkmonitoring']),
                    self.check('properties.tags', {"version": ["v2021-12-01"]}),
                    self.check('properties.body', "heartbeat | take 10")
                    ])

        self.cmd('monitor log-analytics query-pack query list -g {rg} --query-pack-name {pack_name}', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.body', "heartbeat | take 10")
        ])

        self.cmd('monitor log-analytics query-pack query list -g {rg} --query-pack-name {pack_name} --include-body false', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.body', None)
        ])

        self.cmd('monitor log-analytics query-pack query search -g {rg} --query-pack-name {pack_name} '
                 '--categories network', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.id', '{query_id}'),
            self.check('[0].properties.related.categories', ['network', 'monitor', 'databases']),
            self.check('[0].properties.related.resourceTypes',
                       ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('[0].properties.related.solutions', ['networkmonitoring']),
            self.check('[0].properties.tags', {"version": ["v2021-12-01"]}),
            self.check('[0].properties.body', "heartbeat | take 10")
        ])

        self.cmd('monitor log-analytics query-pack query search -g {rg} --query-pack-name {pack_name} '
                 '--resource-types microsoft.insights/autoscalesettings microsoft.network/loadbalancers ', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.id', '{query_id}'),
            self.check('[0].properties.related.categories', ['network', 'monitor', 'databases']),
            self.check('[0].properties.related.resourceTypes',
                       ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('[0].properties.related.solutions', ['networkmonitoring']),
            self.check('[0].properties.tags', {"version": ["v2021-12-01"]}),
            self.check('[0].properties.body', "heartbeat | take 10")
        ])

        self.cmd('monitor log-analytics query-pack query search -g {rg} --query-pack-name {pack_name} '
                 '--solutions "[networkmonitoring]"', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.id', '{query_id}'),
            self.check('[0].properties.related.categories', ['network', 'monitor', 'databases']),
            self.check('[0].properties.related.resourceTypes',
                       ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('[0].properties.related.solutions', ['networkmonitoring']),
            self.check('[0].properties.tags', {"version": ["v2021-12-01"]}),
            self.check('[0].properties.body', "heartbeat | take 10")
        ])

        self.cmd('monitor log-analytics query-pack query search -g {rg} --query-pack-name {pack_name} '
                 '--tags version="[v2021-12-01]"', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.id', '{query_id}'),
            self.check('[0].properties.related.categories', ['network', 'monitor', 'databases']),
            self.check('[0].properties.related.resourceTypes',
                       ['microsoft.network/loadbalancers', 'microsoft.insights/autoscalesettings']),
            self.check('[0].properties.related.solutions', ['networkmonitoring']),
            self.check('[0].properties.tags', {"version": ["v2021-12-01"]}),
            self.check('[0].properties.body', "heartbeat | take 10")
        ])

        self.cmd('monitor log-analytics query-pack query delete --ids {id} --yes')
        self.cmd('monitor log-analytics query-pack query list -g {rg} --query-pack-name {pack_name}', checks=[
            self.check('length(@)', 0),
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.core.aaz import has_value
from azure.cli.command_modules.monitor.aaz.latest.monitor.autoscale._create import Create as _AutoScaleCreate


class AutoScaleCreate(_AutoScaleCreate):

    def _output(self, *args, **kwargs):
        from azure.cli.core.aaz import AAZUndefined
        if has_value(self.ctx.vars.instance.properties.name):
            self.ctx.vars.instance.properties.name = AAZUndefined
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long

from azure.cli.core.aaz import AAZStrArg, AAZListArg, register_command
from azure.cli.command_modules.monitor.aaz.latest.monitor.activity_log.alert._update \
    import Update as _ActivityLogAlertUpdate


@register_command("monitor activity-log alert scope remove")
class ActivityLogAlertScopeRemove(_ActivityLogAlertUpdate):
    """Removes scopes from this activity log alert rule.
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.enabled._registered = False
        args_schema.all_of._registered = False
        args_schema.action_groups._registered = False
        args_schema.tags._registered = False
        args_schema.description._registered = False
        args_schema.scopes._registered = False
        args_schema.scope_ui = AAZListArg(
            options=["--scope", "-s"],
            required=True,
            help="The scopes to remove.",
        )
        args_schema.scope_ui.Element = AAZStrArg()
        return args_schema

    def pre_instance_update(self, instance):
        args = self.ctx.args
        new_scopes = set(instance.properties.scopes.to_serialized_data())
        for scope in args.scope_ui.to_serialized_data():
            try:
                new_scopes.remove(scope)
            except KeyError:
                pass
        args.scopes = list(new_scopes)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def get_cognitiveservices_management_client(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.cognitiveservices import CognitiveServicesManagementClient

    return get_mgmt_service_client(cli_ctx, CognitiveServicesManagementClient)


def cf_accounts(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).accounts


def cf_deleted_accounts(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).deleted_accounts


def cf_deployments(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).deployments


def cf_commitment_tiers(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).commitment_tiers


def cf_commitment_plans(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).commitment_plans


def cf_resource_skus(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).resource_skus


def cf_models(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).models


def cf_usages(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).usages


def cf_ai_projects(cli_ctx, command_args):
    """
    Create AI Projects client for data plane operations.
    Similar to keyvault's data plane client factory pattern.
    """
    from azure.ai.projects import AIProjectClient
    from azure.cli.core.commands.client_factory import prepare_client_kwargs_track2
    from azure.cli.core._profile import Profile

    # Get credentials using Azure CLI login
    profile = Profile(cli_ctx=cli_ctx)
    credential, _, _ = profile.get_login_credentials(
        subscription_id=cli_ctx.data.get("subscription_id")
    )

    # Get endpoint from command arguments (similar to keyvault's vault_url)
    account_name = command_args.get("account_name", None)
    endpoint = command_args.get("endpoint", None)
    project = command_args.get("project_name", None)

    # If no explicit endpoint provided, construct from account name
    if not endpoint and account_name:
        # Construct endpoint URL from account name
        # Format: https://{account_name}.cognitiveservices.azure.com
        endpoint = (
            f"https://{account_name}.services.ai.azure.com/api/projects/{project}"
        )

    if not endpoint:
        from azure.cli.core.azclierror import RequiredArgumentMissingError

        raise RequiredArgumentMissingError(
            "Please specify --account-name or --endpoint"
        )

    # Prepare client kwargs with proper logging and telemetry
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)

    # Create and return the AI Projects client
    return AIProjectClient(endpoint=endpoint, credential=credential, **client_kwargs)


def cf_projects(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).projects


def cf_account_connections(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).account_connections


def cf_managed_network_settings(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).managed_network_settings


def cf_managed_network_provisions(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).managed_network_provisions


def cf_outbound_rule(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).outbound_rule


def cf_account_capability_hosts(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).account_capability_hosts


def cf_project_capability_hosts(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).project_capability_hosts


def cf_managed_compute_deployments(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).managed_compute_deployments


def cf_project_connections(cli_ctx, *_):
    return get_cognitiveservices_management_client(cli_ctx).project_connections



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from collections import OrderedDict
from knack.log import get_logger

logger = get_logger(__name__)


def configstore_output_format(result):
    return _output_format(result, _configstore_format_group)


def deleted_configstore_output_format(result):
    return _output_format(result, _deleted_configstore_format_group)


def configstore_identity_format(result):
    return _output_format(result, _configstore_identity_format_group)


def configstore_credential_format(result):
    return _output_format(result, _configstore_credential_format_group)


def keyvalue_entry_format(result):
    return _output_format(result, _keyvalue_entry_format_group)


def featureflag_entry_format(result):
    return _output_format(result, _featureflag_entry_format_group)


def featurefilter_entry_format(result):
    return _output_format(result, _featurefilter_entry_format_group)


def configstore_replica_output_format(result):
    return _output_format(result, _configstore_replica_format_group)


def configstore_snapshot_output_format(result):
    return _output_format(result, _snapshot_output_format_group)


def configstore_nsp_output_format(result):
    return _output_format(result, _configstore_nsp_format_group)


def _output_format(result, format_group):
    if 'value' in result and isinstance(result['value'], list):
        result = result['value']
    obj_list = result if isinstance(result, list) else [result]
    return [format_group(item) for item in obj_list]


def _configstore_format_group(item):
    sku_value = _get_value(item, 'sku')

    try:
        sku = json.loads(sku_value.replace("\'", "\"")).get('name')
    except ValueError:
        logger.debug("No valid sku %s found", sku_value)
        sku = ""

    return OrderedDict([
        ('CREATION DATE', _format_datetime(_get_value(item, 'creationDate'))),
        ('ENDPOINT', _get_value(item, 'endpoint')),
        ('LOCATION', _get_value(item, 'location')),
        ('NAME', _get_value(item, 'name')),
        ('PROVISIONING STATE', _get_value(item, 'provisioningState')),
        ('RESOURCE GROUP', _get_value(item, 'resourceGroup')),
        ('SKU', sku),
        ('PURGE PROTECTION', 'ENABLED' if _get_value(item, 'enablePurgeProtection').lower() == 'true' else 'DISABLED'),
        ('SOFT DELETE RETENTION PERIOD', _get_value(item, 'softDeleteRetentionInDays') + ' DAYS')
    ])


def _deleted_configstore_format_group(item):
    return OrderedDict([
        ('DELETION DATE', _format_datetime(_get_value(item, 'deletionDate'))),
        ('LOCATION', _get_value(item, 'location')),
        ('NAME', _get_value(item, 'name')),
        ('PURGE PROTECTION', 'ENABLED' if _get_value(item, 'purgeProtectionEnabled').lower() == 'true' else 'DISABLED'),
        ('SCHEDULED PURGE DATE', _format_datetime(_get_value(item, 'scheduledPurgeDate')))
    ])


def _configstore_replica_format_group(item):
    return OrderedDict([
        ('CREATION DATE', _format_datetime(_get_value(item, 'systemData', 'createdAt'))),
        ('ENDPOINT', _get_value(item, 'endpoint')),
        ('LOCATION', _get_value(item, 'location')),
        ('NAME', _get_value(item, 'name')),
        ('PROVISIONING STATE', _get_value(item, 'provisioningState')),
        ('RESOURCE GROUP', _get_value(item, 'resourceGroup'))
    ])


def _configstore_identity_format_group(item):
    return OrderedDict([
        ('PRINCIPAL ID', _format_datetime(_get_value(item, 'principalId'))),
        ('TENANT ID', _get_value(item, 'tenantId')),
        ('TYPE', _get_value(item, 'type'))
    ])


def _configstore_credential_format_group(item):
    return OrderedDict([
        ('CONNECTION STRING', _get_value(item, 'connectionString')),
        ('ID', _get_value(item, 'id')),
        ('LAST MODIFIED', _format_datetime(_get_value(item, 'lastModified'))),
        ('NAME', _get_value(item, 'name')),
        ('READ ONLY', _get_value(item, 'readOnly')),
        ('VALUE', _get_value(item, 'value'))
    ])


def _keyvalue_entry_format_group(item):

    return OrderedDict([
        ('CONTENT TYPE', _get_value_by_names(item, ['contentType', 'content_type'])),
        ('KEY', _get_value(item, 'key')),
        ('VALUE', _get_value(item, 'value')),
        ('LAST MODIFIED', _format_datetime(_get_value_by_names(item, ['lastModified', 'last_modified']))),
        ('TAGS', _get_value(item, 'tags')),
        ('LABEL', _get_value(item, 'label')),
        ('LOCKED', _get_value(item, 'locked'))
    ])


def _featureflag_entry_format_group(item):

    return OrderedDict([
        ('KEY', _get_value(item, 'key')),
        ('LABEL', _get_value(item, 'label')),
        ('STATE', _get_value(item, 'state')),
        ('LOCKED', _get_value(item, 'locked')),
        ('DESCRIPTION', _get_value(item, 'description')),
        ('LAST MODIFIED', _format_datetime(_get_value_by_names(item, ['lastModified', 'last_modified']))),
        ('CONDITIONS', _get_value(item, 'conditions'))
    ])


def _snapshot_output_format_group(item):

    return OrderedDict([
        ('NAME', _get_value(item, 'name')),
        ('FILTERS', _get_value(item, 'filters')),
        ('COMPOSITION TYPE', _get_value_by_names(item, ['compositionType', 'composition_type'])),
        ('CREATED', _format_datetime(_get_value(item, 'created'))),
        ('EXPIRES', _format_datetime(_get_value(item, 'expires'))),
        ('RETENTION PERIOD', _get_value_by_names(item, ['retentionPeriod', 'retention_period'])),
        ('SIZE', _get_value(item, 'size')),
        ('STATUS', _get_value(item, 'status')),
        ('ITEMS COUNT', _get_value_by_names(item, ['itemsCount', 'items_count'])),
        ('TAGS', _get_value(item, 'tags')),
    ])


def _featurefilter_entry_format_group(item):
    return OrderedDict([
        ('NAME', _get_value(item, 'name')),
        ('PARAMETERS', _get_value(item, 'parameters'))
    ])


def _configstore_nsp_format_group(item):
    return OrderedDict([
        ('PERIMETER ID', _get_value(item, 'properties', 'networkSecurityPerimeter', 'id')),
        ('PROFILE NAME', _get_value(item, 'properties', 'profile', 'name')),
        ('ACCESS MODE', _get_value(item, 'properties', 'resourceAssociation', 'accessMode')),
        ('ACCESS RULES VERSION', _get_value(item, 'properties', 'profile', 'accessRulesVersion')),
        ('DIAGNOSTIC SETTINGS VERSION', _get_value(item, 'properties', 'profile', 'diagnosticSettingsVersion'))
    ])


def _format_datetime(date_string):
    from dateutil.parser import parse
    try:
        return parse(date_string).strftime("%Y-%m-%dT%H:%M:%SZ")
    except ValueError:
        return date_string or ' '


def _get_value(item, *args):
    """Recursively get a nested value from a dict.
    :param dict item: The dict object
    """
    try:
        for arg in args:
            item = item[arg]
        return _EvenLadder(item) if item is not None else ' '
    except (KeyError, TypeError, IndexError):
        return ' '


def _get_value_by_names(item, property_names):
    """
        CLI core converts KeyValue object field names to camelCase (eg: content_type becomes contentType)
        But when customers specify field filters, we return a dict of requested fields instead of KeyValue object
        In that case, field names are not converted to camelCase. We need to check for both content_type and contentType
        We iterate through given names and return the first value that is found
    """
    # pylint: disable=line-too-long
    # This is the value that will be returned if no value is found or the value is None. This maintains consistency with the default table transformer logic
    # https://github.com/Azure/azure-cli/blob/fdb9aa742b404a433f207b573eea37341769020c/src/azure-cli-core/azure/cli/core/commands/transform.py#L91C1-L98C30
    empty_value = ' '

    property_value = empty_value

    for property_name in property_names:
        property_value = _get_value(item, property_name)
        if property_value != empty_value:
            break
    return property_value


def _EvenLadder(item):
    formated_item = ''
    item = str(item)
    if len(item) < 70:
        return item
    for i in range(0, len(item), 70):
        formated_item += str(item[i: i + 70]) + "\n"
    return formated_item



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer

LOCATION = "eastus"
VNET_LOCATION = "eastus"
VOLUME_DEFAULT = "--service-level 'Premium' --usage-threshold 100"
vnet_name = "cli-vnet-lefr-02"
subnet_name = "default"


class AzureNetAppFilesSnapshotPolicyServiceScenarioTest(ScenarioTest):
    def setup_vnet(self, vnet_name, subnet_name):
        self.cmd("az network vnet create -n %s -g {rg} -l %s --address-prefix 10.5.0.0/16" %
                 (vnet_name, VNET_LOCATION))
        self.cmd("az network vnet subnet create -n %s --vnet-name %s --address-prefixes '10.5.0.0/24' "
                 "--delegations 'Microsoft.Netapp/volumes' -g {rg}" % (subnet_name, vnet_name))

    def current_subscription(self):
        subs = self.cmd("az account show").get_output_in_json()
        return subs['id']

    def create_volume(self, account_name, pool_name, volume_name, snapshot_policy_id=None, volume_only=False):
        if not volume_only:
            # create vnet and pool
            self.setup_vnet(vnet_name, subnet_name)
            self.cmd("netappfiles pool create -g {rg} -a %s -p %s -l %s --service-level 'Premium' --size 4" %
                     (account_name, pool_name, LOCATION))

        snapshot = "--snapshot-policy-id %s" % snapshot_policy_id if snapshot_policy_id is not None else ""

        # create volume
        return self.cmd("netappfiles volume create -g {rg} -a %s -p %s -v %s -l %s --vnet %s --subnet %s "
                        "--file-path %s %s %s" %
                        (account_name, pool_name, volume_name, LOCATION, vnet_name, subnet_name, volume_name,
                         VOLUME_DEFAULT, snapshot)).get_output_in_json()

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_create_delete_snapshot_policies(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        snapshot_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create snapshot policy using long parameter names and validate result
        hourly_snapshots_to_keep = 1
        hourly_minute = 10
        daily_snapshots_to_keep = 2
        daily_minute = 20
        daily_hour = 2
        weekly_snapshots_to_keep = 3
        weekly_minute = 30
        weekly_hour = 3
        weekly_day = "Monday"
        monthly_snapshots_to_keep = 4
        monthly_minute = 40
        monthly_hour = 4
        monthly_days_of_month = "1,3,20"
        enabled = True
        tags = "Tag1=Value1"
        snapshot_policy = self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s "
                                   "--location %s --hourly-snapshots %s --daily-snapshots %s "
                                   "--weekly-snapshots %s --monthly-snapshots %s --hourly-minute %s "
                                   "--daily-minute %s --weekly-minute %s --monthly-minute %s --daily-hour %s "
                                   "--weekly-hour %s --monthly-hour %s --weekly-day %s --monthly-days %s "
                                   "--enabled %s --tags %s" %
                                   (account_name, snapshot_policy_name, LOCATION, hourly_snapshots_to_keep,
                                    daily_snapshots_to_keep, weekly_snapshots_to_keep, monthly_snapshots_to_keep,
                                    hourly_minute, daily_minute, weekly_minute, monthly_minute, daily_hour, weekly_hour,
                                    monthly_hour, weekly_day, monthly_days_of_month, enabled, tags)).get_output_in_json()
        assert snapshot_policy['name'] == account_name + "/" + snapshot_policy_name
        assert snapshot_policy['hourlySchedule']['snapshotsToKeep'] == hourly_snapshots_to_keep
        assert snapshot_policy['hourlySchedule']['minute'] == hourly_minute
        assert snapshot_policy['dailySchedule']['snapshotsToKeep'] == daily_snapshots_to_keep
        assert snapshot_policy['dailySchedule']['minute'] == daily_minute
        assert snapshot_policy['dailySchedule']['hour'] == daily_hour
        assert snapshot_policy['weeklySchedule']['snapshotsToKeep'] == weekly_snapshots_to_keep
        assert snapshot_policy['weeklySchedule']['minute'] == weekly_minute
        assert snapshot_policy['weeklySchedule']['hour'] == weekly_hour
        assert snapshot_policy['weeklySchedule']['day'] == weekly_day
        assert snapshot_policy['monthlySchedule']['snapshotsToKeep'] == monthly_snapshots_to_keep
        assert snapshot_policy['monthlySchedule']['minute'] == monthly_minute
        assert snapshot_policy['monthlySchedule']['hour'] == monthly_hour
        assert snapshot_policy['monthlySchedule']['daysOfMonth'] == monthly_days_of_month
        assert snapshot_policy['enabled'] == enabled
        assert snapshot_policy['tags']['Tag1'] == 'Value1'
        assert snapshot_policy['provisioningState'] is not None

        # validate snapshot policy exist
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                                   (account_name, snapshot_policy_name)).get_output_in_json()
        assert snapshot_policy['tags']['Tag1'] == 'Value1'

        # validate snapshot policy exist via list
        snapshot_policy_list = self.cmd("az netappfiles snapshot policy list -g {rg} -a '%s'" %
                                        account_name).get_output_in_json()
        assert len(snapshot_policy_list) == 1

        # delete snapshot policy
        self.cmd("az netappfiles snapshot policy delete -g {rg} -a %s --snapshot-policy-name %s -y" %
                 (account_name, snapshot_policy_name))

        # create snapshot policy using short parameter names and validate result
        snapshot_policy = self.cmd("az netappfiles snapshot policy create -g {rg} -a %s "
                                   "--snapshot-policy-name %s -l %s -u %s -d %s -w %s -m %s "
                                   "--hourly-minute %s --daily-minute %s --weekly-minute %s --monthly-minute %s "
                                   "--daily-hour %s --weekly-hour %s --monthly-hour %s --weekly-day %s "
                                   "--monthly-days %s --enabled %s --tags %s" %
                                   (account_name, snapshot_policy_name, LOCATION, hourly_snapshots_to_keep,
                                    daily_snapshots_to_keep, weekly_snapshots_to_keep, monthly_snapshots_to_keep,
                                    hourly_minute, daily_minute, weekly_minute, monthly_minute, daily_hour, weekly_hour,
                                    monthly_hour, weekly_day, monthly_days_of_month, enabled, tags)).get_output_in_json()
        assert snapshot_policy['name'] == account_name + "/" + snapshot_policy_name
        assert snapshot_policy['hourlySchedule']['snapshotsToKeep'] == hourly_snapshots_to_keep
        assert snapshot_policy['hourlySchedule']['minute'] == hourly_minute
        assert snapshot_policy['dailySchedule']['snapshotsToKeep'] == daily_snapshots_to_keep
        assert snapshot_policy['dailySchedule']['minute'] == daily_minute
        assert snapshot_policy['dailySchedule']['hour'] == daily_hour
        assert snapshot_policy['weeklySchedule']['snapshotsToKeep'] == weekly_snapshots_to_keep
        assert snapshot_policy['weeklySchedule']['minute'] == weekly_minute
        assert snapshot_policy['weeklySchedule']['hour'] == weekly_hour
        assert snapshot_policy['weeklySchedule']['day'] == weekly_day
        assert snapshot_policy['monthlySchedule']['snapshotsToKeep'] == monthly_snapshots_to_keep
        assert snapshot_policy['monthlySchedule']['minute'] == monthly_minute
        assert snapshot_policy['monthlySchedule']['hour'] == monthly_hour
        assert snapshot_policy['monthlySchedule']['daysOfMonth'] == monthly_days_of_month
        assert snapshot_policy['enabled'] == enabled
        assert snapshot_policy['tags']['Tag1'] == 'Value1'

        # delete snapshot policy
        self.cmd("az netappfiles snapshot policy delete -g {rg} -a %s --snapshot-policy-name %s -y" %
                 (account_name, snapshot_policy_name))

        # validate snapshot policy doesn't exist
        snapshot_policy_list = self.cmd("az netappfiles snapshot policy list -g {rg} -a '%s'" %
                                        account_name).get_output_in_json()
        assert len(snapshot_policy_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_list_snapshot_policy(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create 3 snapshot policies
        snapshot_policies = [self.create_random_name(prefix='cli', length=16),
                             self.create_random_name(prefix='cli', length=16),
                             self.create_random_name(prefix='cli', length=16)]
        hourly_snapshots_to_keep = 1
        hourly_minute = 10
        for snapshot_policy_name in snapshot_policies:
            self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s -l %s -u %s --hourly-minute %s" %
                     (account_name, snapshot_policy_name, LOCATION, hourly_snapshots_to_keep, hourly_minute))

        # validate that both snapshot policies exist
        snapshot_policy_list = self.cmd("az netappfiles snapshot policy list -g {rg} -a '%s'" %
                                        account_name).get_output_in_json()
        assert len(snapshot_policy_list) == 3

        # delete all snapshot policies
        for snapshot_policy_name in snapshot_policies:
            self.cmd("az netappfiles snapshot policy delete -g {rg} -a %s --snapshot-policy-name %s -y" %
                     (account_name, snapshot_policy_name))

        # validate that no snapshot policies exist
        snapshot_policy_list = self.cmd("az netappfiles snapshot policy list -g {rg} -a '%s'" %
                                        account_name).get_output_in_json()
        assert len(snapshot_policy_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_get_snapshot_policy_by_name(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create snapshot policy
        snapshot_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        hourly_snapshots = 1
        hourly_minute = 10
        daily_minute = 20
        daily_hour = 2
        self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s -l %s -u %s --hourly-minute %s --daily-minute %s --daily-hour %s" %
                 (account_name, snapshot_policy_name, LOCATION, hourly_snapshots, hourly_minute, daily_minute, daily_hour)).get_output_in_json()

        # get snapshot policy by name and validate
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                                   (account_name, snapshot_policy_name)).get_output_in_json()
        assert snapshot_policy['name'] == account_name + '/' + snapshot_policy_name
        assert snapshot_policy['hourlySchedule']['snapshotsToKeep'] == hourly_snapshots
        assert snapshot_policy['hourlySchedule']['minute'] == hourly_minute

        # get snapshot policy by resource id and validate
        snapshot_policy_from_id = self.cmd("az netappfiles snapshot policy show --ids %s" %
                                           snapshot_policy['id']).get_output_in_json()
        assert snapshot_policy_from_id['name'] == account_name + '/' + snapshot_policy_name
        assert snapshot_policy['hourlySchedule']['snapshotsToKeep'] == hourly_snapshots
        assert snapshot_policy['hourlySchedule']['minute'] == hourly_minute

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_update_snapshot_policy(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create snapshot policy
        snapshot_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        hourly_snapshots_to_keep = 1
        hourly_minute = 10
        daily_snapshots_to_keep = 2
        daily_minute = 20
        daily_hour = 2
        weekly_snapshots_to_keep = 3
        weekly_minute = 30
        weekly_hour = 3
        weekly_day = "Monday"
        monthly_snapshots_to_keep = 4
        monthly_minute = 40
        monthly_hour = 4
        monthly_days_of_month = "2,5,30"
        enabled = True
        tags = "Tag1=Value1"
        snapshot_policy =self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s -l %s -u %s -d %s -w %s -m %s "
                 "--hourly-minute %s --daily-minute %s --weekly-minute %s --monthly-minute %s --daily-hour %s "
                 "--weekly-hour %s --monthly-hour %s --weekly-day %s --monthly-days %s --enabled %s --tags %s" %
                 (account_name, snapshot_policy_name, LOCATION, hourly_snapshots_to_keep,
                  daily_snapshots_to_keep, weekly_snapshots_to_keep, monthly_snapshots_to_keep,
                  hourly_minute, daily_minute, weekly_minute, monthly_minute, daily_hour, weekly_hour,
                  monthly_hour, weekly_day, monthly_days_of_month, enabled, tags)).get_output_in_json()
        assert snapshot_policy['tags']['Tag1'] == 'Value1'

        #validate created
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                            (account_name, snapshot_policy_name)).get_output_in_json()
        assert snapshot_policy['tags']['Tag1'] == 'Value1'

        # update snapshot policy
        hourly_snapshots_to_keep = 5
        hourly_minute = 50
        daily_snapshots_to_keep = 6
        daily_minute = 0
        daily_hour = 6
        weekly_snapshots_to_keep = 7
        weekly_minute = 10
        weekly_hour = 7
        weekly_day = "Wednesday"
        monthly_snapshots_to_keep = 8
        monthly_minute = 20
        monthly_hour = 8
        monthly_days_of_month = "1,2,20"
        enabled = False
        tags = "Tag1=Value2"
        snapshot_policy = self.cmd("az netappfiles snapshot policy update -g {rg} -a %s --snapshot-policy-name %s -u %s -d %s -w %s -m %s "
                 "--hourly-minute %s --daily-minute %s --weekly-minute %s --monthly-minute %s --daily-hour %s "
                 "--weekly-hour %s --monthly-hour %s --weekly-day %s --monthly-days %s --enabled %s --tags %s" %
                 (account_name, snapshot_policy_name, hourly_snapshots_to_keep,
                  daily_snapshots_to_keep, weekly_snapshots_to_keep, monthly_snapshots_to_keep,
                  hourly_minute, daily_minute, weekly_minute, monthly_minute, daily_hour, weekly_hour,
                  monthly_hour, weekly_day, monthly_days_of_month, enabled, tags)).get_output_in_json()
        # assert snapshot_policy['tags']['Tag1'] == 'Value2'

        # get updated snapshot policy and validate update
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                                   (account_name, snapshot_policy_name)).get_output_in_json()
        assert snapshot_policy['name'] == account_name + "/" + snapshot_policy_name
        assert snapshot_policy['hourlySchedule']['snapshotsToKeep'] == hourly_snapshots_to_keep
        assert snapshot_policy['hourlySchedule']['minute'] == hourly_minute
        assert snapshot_policy['dailySchedule']['snapshotsToKeep'] == daily_snapshots_to_keep
        assert snapshot_policy['dailySchedule']['minute'] == daily_minute
        assert snapshot_policy['dailySchedule']['hour'] == daily_hour
        assert snapshot_policy['weeklySchedule']['snapshotsToKeep'] == weekly_snapshots_to_keep
        assert snapshot_policy['weeklySchedule']['minute'] == weekly_minute
        assert snapshot_policy['weeklySchedule']['hour'] == weekly_hour
        assert snapshot_policy['weeklySchedule']['day'] == weekly_day
        assert snapshot_policy['monthlySchedule']['snapshotsToKeep'] == monthly_snapshots_to_keep
        assert snapshot_policy['monthlySchedule']['minute'] == monthly_minute
        assert snapshot_policy['monthlySchedule']['hour'] == monthly_hour
        assert snapshot_policy['monthlySchedule']['daysOfMonth'] == monthly_days_of_month
        assert snapshot_policy['enabled'] == enabled
        assert snapshot_policy['tags']['Tag1'] == 'Value2'
        #validate created
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                            (account_name, snapshot_policy_name)).get_output_in_json()
        assert snapshot_policy['tags']['Tag1'] == 'Value2'


    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_snapshot_policy_list_volumes(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION))

        # create snapshot policy
        snapshot_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        hourly_snapshots_to_keep = 1
        hourly_minute = 10
        enabled = True
        self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s -l %s -u %s --hourly-minute %s --enabled %s"
                 % (account_name, snapshot_policy_name, LOCATION, hourly_snapshots_to_keep, hourly_minute, enabled))

        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                                   (account_name, snapshot_policy_name)).get_output_in_json()

        # create volume
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        self.create_volume(account_name, pool_name, volume_name, snapshot_policy_id=snapshot_policy['id'])

        list_volumes = self.cmd("az netappfiles snapshot policy volumes -g {rg} -a %s --snapshot-policy-name %s" %
                                (account_name, snapshot_policy_name)).get_output_in_json()

        assert len(list_volumes['value']) == 1

        # create second volume
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        self.create_volume(account_name, pool_name, volume_name, snapshot_policy_id=snapshot_policy['id'],
                           volume_only=True)

        list_volumes = self.cmd("az netappfiles snapshot policy volumes -g {rg} -a %s --snapshot-policy-name %s" %
                                (account_name, snapshot_policy_name)).get_output_in_json()

        assert len(list_volumes['value']) == 2

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_snapshot_policy_', additional_tags={'owner': 'cli_test'})
    def test_assign_snapshot_policy_to_volume(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION))

        # create volume
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        self.create_volume(account_name, pool_name, volume_name)

        # create snapshot policy
        snapshot_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        hourly_snapshots_to_keep = 1
        hourly_minute = 10
        enabled = True
        self.cmd("az netappfiles snapshot policy create -g {rg} -a %s --snapshot-policy-name %s -l %s -u %s "
                 "--hourly-minute %s --enabled %s" % (account_name, snapshot_policy_name, LOCATION,
                                                      hourly_snapshots_to_keep, hourly_minute, enabled))
        snapshot_policy = self.cmd("az netappfiles snapshot policy show -g {rg} -a %s --snapshot-policy-name %s" %
                                   (account_name, snapshot_policy_name)).get_output_in_json()

        # assign snapshot policy to the volume
        self.cmd("az netappfiles volume update -g {rg} -a %s -p %s -v %s --snapshot-policy-id %s" %
                 (account_name, pool_name, volume_name, snapshot_policy['id']))

        volume = self.cmd("az netappfiles volume show -g {rg} -a %s -p %s -v %s" %
                          (account_name, pool_name, volume_name)).get_output_in_json()
        assert volume['dataProtection']['snapshot']['snapshotPolicyId'] == snapshot_policy['id']



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# AZURE CLI EventHub - NAMESPACE TEST DEFINITIONS

import time

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, KeyVaultPreparer)


# pylint: disable=line-too-long
# pylint: disable=too-many-lines


class EHNamespaceAUTHRULECURDScenarioTest(ScenarioTest):
    from azure.cli.testsdk.scenario_tests import AllowLargeResponse

    @AllowLargeResponse()
    @ResourceGroupPreparer(name_prefix='cli_test_eh_namespace')
    def test_eh_authorization_rule(self, resource_group):
        self.kwargs.update({
            'loc': 'southcentralus',
            'rg': resource_group,
            'namespacename': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'eventhub1': self.create_random_name(prefix='eventhubnm', length=18),
            'namespacenamekafka': self.create_random_name(prefix='eventhubs-nscli1', length=20),
            'tags': {'tag1=value1'},
            'tags2': {'tag2=value2'},
            'sku': 'Standard',
            'tier': 'Standard',
            'clusterarmid': '/subscriptions/326100e2-f69d-4268-8503-075374f62b6e/resourceGroups/v-ajnavtest/providers/Microsoft.EventHub/clusters/PMTestCluster',
            'authoname': self.create_random_name(prefix='cliAutho', length=20),
            'defaultauthorizationrule': 'RootManageSharedAccessKey',
            'accessrights': 'Send',
            'accessrights1': 'Listen',
            'primary': 'PrimaryKey',
            'secondary': 'SecondaryKey',
            'isautoinflateenabled': 'True',
            'enableidentity': 'True',
            'maximumthroughputunits': 4,
            'maximumthroughputunits_update': 5

        })
        # Check for the NameSpace name Availability
        self.cmd('eventhubs namespace exists --name {namespacename}', checks=[self.check('nameAvailable', True)])

        # Create Namespace
        self.cmd('eventhubs namespace create --resource-group {rg} --name {namespacename} --location {loc} --tags {tags} --sku Standard')

        # Create Authorization Rule
        authRule = self.cmd(
            'eventhubs namespace authorization-rule create --resource-group {rg} --namespace-name {namespacename} --name namespaceAuthRule1 --rights Listen Send Manage').get_output_in_json()
        self.assertEqual(authRule['name'], 'namespaceAuthRule1')
        n = [i for i in authRule['rights']]
        assert len(n) == 3

        # Update Authorization Rule
        authRule = self.cmd(
            'eventhubs namespace authorization-rule update --resource-group {rg} --namespace-name {namespacename} --name namespaceAuthRule1 --rights Listen').get_output_in_json()
        self.assertEqual(authRule['name'], 'namespaceAuthRule1')
        n = [i for i in authRule['rights']]
        assert len(n) == 1

        # Get Authorization Rule List
        listOfAuthRules = self.cmd(
            'eventhubs namespace authorization-rule list --resource-group {rg} --namespace-name {namespacename}').get_output_in_json()
        assert len(listOfAuthRules) == 2

        # Get Authorization Rule Keys List
        currentKeys = self.cmd(
            'eventhubs namespace authorization-rule keys list --resource-group {rg} --namespace-name {namespacename} --name namespaceAuthRule1').get_output_in_json()

        # Regeneratekeys - Primary
        regenerateprimarykeyresult = self.cmd(
            'eventhubs namespace authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --name namespaceAuthRule1 --key {primary}').get_output_in_json()
        self.assertIsNotNone(regenerateprimarykeyresult)
        self.assertNotEqual(currentKeys['primaryKey'], regenerateprimarykeyresult['primaryKey'])
        self.assertEqual(currentKeys['secondaryKey'], regenerateprimarykeyresult['secondaryKey'])

        currentKeys = regenerateprimarykeyresult

        # Regeneratekeys - Secondary
        regeneratesecondarykeyresult = self.cmd(
            'eventhubs namespace authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --name namespaceAuthRule1 --key {secondary}').get_output_in_json()
        self.assertIsNotNone(regeneratesecondarykeyresult)
        self.assertEqual(currentKeys['primaryKey'], regeneratesecondarykeyresult['primaryKey'])
        self.assertNotEqual(currentKeys['secondaryKey'], regeneratesecondarykeyresult['secondaryKey'])

        #create Eventhub entity
        self.cmd('eventhubs eventhub create --resource-group {rg} --namespace-name {namespacename} --name {eventhub1}')

        # Create Authorization Rule for eventhub
        authRule = self.cmd(
            'eventhubs eventhub authorization-rule create --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhub1} --name eventHubAuthRule1 --rights Listen Send Manage').get_output_in_json()
        self.assertEqual(authRule['name'], 'eventHubAuthRule1')
        n = [i for i in authRule['rights']]
        assert len(n) == 3

        # Get Authorization Rule
        authRule = self.cmd(
            'eventhubs eventhub authorization-rule show --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhub1} --name eventHubAuthRule1').get_output_in_json()
        self.assertEqual(authRule['name'], 'eventHubAuthRule1')
        n = [i for i in authRule['rights']]
        assert len(n) == 3

        # Get Authorization Rule Listkeys
        currentKeys = self.cmd(
            'eventhubs eventhub authorization-rule keys list --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhub1} --name eventHubAuthRule1').get_output_in_json()

        # Regeneratekeys - Primary
        regenerateprimarykeyresult = self.cmd(
            'eventhubs eventhub authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhub1} --name eventHubAuthRule1 --key {primary}').get_output_in_json()
        self.assertIsNotNone(regenerateprimarykeyresult)
        self.assertNotEqual(currentKeys['primaryKey'], regenerateprimarykeyresult['primaryKey'])
        self.assertEqual(currentKeys['secondaryKey'], regenerateprimarykeyresult['secondaryKey'])

        currentKeys = regenerateprimarykeyresult

        # Regeneratekeys - Secondary
        regeneratesecondarykeyresult = self.cmd(
            'eventhubs eventhub authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhub1} --name eventHubAuthRule1 --key {secondary}').get_output_in_json()
        self.assertIsNotNone(regeneratesecondarykeyresult)
        self.assertEqual(currentKeys['primaryKey'], regeneratesecondarykeyresult['primaryKey'])
        self.assertNotEqual(currentKeys['secondaryKey'], regeneratesecondarykeyresult['secondaryKey'])

        # Delete Namespace list by ResourceGroup
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# The level of complex object namespace to flatten.
FLATTEN = 3

# Non-complex types that should not be flattened.
BASIC_TYPES = {
    'str',
    'int',
    'bool',
    'float',
    'long',
    'duration',
    'iso-8601',
    'rfc-1123',
    'date',
    'decimal',
    'unix-time',
    'datetime.datetime',
    'datetime.timedelta'
}

# Parameters that we want to be present in the namespace, but not exposed to the user as arguments
SILENT_PARAMETERS = [
    'pool.virtual_machine_configuration.image_reference',
    'pool.virtual_machine_configuration.disk_encryption_configuration'
]

# Common argument names the should always be prefixed by their context
QUALIFIED_PROPERTIES = [
    'id',
    'display_name',
    'command_line',
    'environment_settings',
    'wait_for_success',
    'max_task_retry_count',
    'constraints_max_task_retry_count',
    'max_wall_clock_time',
    'constraints_max_wall_clock_time',
    'retention_time',
    'constraints_retention_time',
    'application_package_references',
    'resource_files',
    'user_name'
]

# Header options that should not be exposed as arguments.
IGNORE_OPTIONS = {
    'ocp_date',
    'timeout',
    'client_request_id',
    'return_client_request_id',
    'max_results',
    'additional_properties'
}

# Parameters that can be completely ignored.
# This can be the complete path to a single parameter to be removed, or any part
# of a path. Any parameter that has a match to the partial path will be removed.
IGNORE_PARAMETERS = [
    'callback',
    'thumbprint_algorithm',
    'display_name',
    'common_environment_settings',
    'job_preparation_task',
    'job_release_task',
    'auto_pool_specification',
    'on_task_failure',
    # 'job.on_all_tasks_complete',
    'job_manager_task.kill_job_on_completion',
    'job_manager_task.run_exclusive',
    'job_manager_task.constraints',
    'job_manager_task.allow_low_priority_node',
    # 'job.job_manager_task.application_package_references',
    'virtual_machine_configuration.windows_configuration',
    'virtual_machine_configuration.container_configuration',
    'virtual_machine_configuration.license_type',
    'task_scheduling_policy',
    'container_settings',
    'user_identity',
    'network_configuration.dynamic_vnet_assignment_scope',
    'network_configuration.dynamic_v_net_assignment_scope',
    'network_configuration.public_ip_address_configuration',
    'network_configuration.subnet_id',
    'enable_auto_scale',
    'cloud_service_configuration.target_os_version',
    'pool.auto_scale_evaluation_interval',
    # 'pool.start_task.environment_settings',
    # 'pool.start_task.max_task_retry_count',
    'job_schedule_patch_parameter.job_specification.constraints',
    'job_schedule_update_parameter.job_specification.constraints',
    # 'job_schedule.job_specification.metadata',
    # 'job_schedule.job_specification.job_manager_task.application_package_references',
    # 'job_schedule.job_specification.job_manager_task.environment_settings',
    'resource_file.storage_container_url',
    'resource_file.blob_prefix',
    'resource_file.auto_storage_container_name',
    'virtual_machine_configuration.service_artifact_reference',
    'target_node_communication_mode'
]

# Options to be flattened into multiple arguments.
FLATTEN_OPTIONS = {
    'ocp_range': {
        'start_range': "The byte range to be retrieved. If not set the file will be retrieved from the beginning.",
        'end_range': "The byte range to be retrieved. If not set the file will be retrieved to the end."
    }
}
